// This file is part of the aMule Project
//
// Copyright (c) 2003-2004 aMule Project ( http://www.amule-project.net )
// Copyright (C) 2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "sys.h"

#include <wx/defs.h>		// Needed before any other wx/*.h
#include "CString.h"		// Interface declarations.

#include <wx/listimpl.cpp>	// Needed for WX_DEFINE_LIST
_DEFINE_LIST(char*, VoidList);	// WX_DEFINE_LIST(VoidList), but avoiding warning.
WX_DEFINE_LIST(stringList);

int stricmp( const char* s1, const char* s2 ) {
	return(wxString::wxString(s1).CmpNoCase(s2));
}

