// This file is part of the aMule Project
//
// Copyright (c) 2003-2004 aMule Project ( http://www.amule-project.net )
// Copyright (C) 2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef WINTYPES_H
#define WINTYPES_H

#include <inttypes.h>		// Needed for uint32_t and uint16_t

typedef unsigned int		UINT;
typedef UINT*			UINT_PTR;

typedef char*			LPCSTR;
typedef char*			LPSTR;
typedef char*			LPCTSTR;
typedef unsigned char		BYTE;
typedef uint32_t		DWORD;
typedef long			LONG;
typedef unsigned long		ULONG;
typedef unsigned long long	ULONGLONG;
typedef BYTE*			LPBYTE;
typedef int			INT;
typedef uint16_t		WORD;
typedef void			VOID;
typedef void*			LPVOID;
typedef void*			PVOID;
typedef char			TCHAR;

struct POINT {
  UINT x;
  UINT y;
};

struct RECT {
  UINT left;
  UINT top;
  UINT right;
  UINT bottom;
};

typedef RECT*			LPRECT;

struct WINDOWPLACEMENT {
  UINT length;
  UINT flags;
  UINT showCmd;
  POINT ptMinPosition;
  POINT ptMaxPosition;
  RECT rcNormalPosition;
};

typedef unsigned short          WPARAM;
typedef unsigned long           LPARAM;
typedef unsigned char		byte;

#endif // WINTYPES_H
