#ifndef RESOURCE_H
#define RESOURCE_H

// Microsoft Visual C++ generated include file.
// Used by amule.rc
//
#define IDR_MANIFEST                    1
#define IDC_BUTTONRENAME                2
//#define IDC_BUTTONSTRIP                 3
#define IDM_ABOUTBOX                    0x0010
#define IDM_VERSIONCHECK                0x0011
#define IDC_STATUSBAR                   95
#define IDD_ABOUTBOX                    96
#define IDD_EMULE_DIALOG                97
#define IDR_HTML_PAGESELECTIONBOX       98
#define IDP_SOCKETS_INIT_FAILED         101
#define IDS_ABOUTBOX                    102
#define IDS_HTTPDOWNLOAD_FILESTATUS     103
#define IDS_HTTPDOWNLOAD_CONNECTED      104
#define IDS_HTTPDOWNLOAD_RESOLVING_NAME 105
#define IDS_HTTPDOWNLOAD_RESOLVED_NAME  106
#define IDS_HTTPDOWNLOAD_CONNECTING     107
#define IDS_HTTPDOWNLOAD_REDIRECTING    108
#define IDS_HTTPDOWNLOAD_GETTING_FILE_INFORMATION 109
#define IDS_HTTPDOWNLOAD_FAIL_PARSE_ERROR 110
#define IDS_HTTPDOWNLOAD_GENERIC_ERROR  111
#define IDS_HTTPDOWNLOAD_FAIL_CONNECT_SERVER 112
#define IDS_HTTPDOWNLOAD_INVALID_SERVER_RESPONSE 113
#define IDS_HTTPDOWNLOAD_INVALID_HTTP_RESPONSE 114
#define IDS_HTTPDOWNLOAD_ERROR_READFILE 115
#define IDS_HTTPDOWNLOAD_PERCENTAGE     116
#define IDS_HTTPDOWNLOAD_RETREIVEING_FILE 117
#define IDS_HTTPDOWNLOAD_OF             118
#define IDS_HTTPDOWNLOAD_SECONDS        119
#define IDS_HTTPDOWNLOAD_MINUTES        120
#define IDS_HTTPDOWNLOAD_MINUTES_AND_SECONDS 121
#define IDS_HTTPDOWNLOAD_BYTES          122
#define IDS_HTTPDOWNLOAD_KILOBYTES      123
#define IDS_CANCEL                      124
#define IDS_HTTPDOWNLOAD_MEGABYTES      125
#define IDS_HTTP_CAPTION                126
#define IDS_HTTPDOWNLOAD_TIMELEFT       127
#define IDS_HTTPDOWNLOAD_BYTESPERSECOND 128
#define IDR_MAINFRAME                   129
#define IDS_HTTPDOWNLOAD_KILOBYTESPERSECOND 129
#define IDD_TRANSFER                    130
#define IDS_HTTPDOWNLOAD_OK_TO_OVERWRITE 130
#define IDD_SERVER                      131
#define IDS_HTTPDOWNLOAD_FAIL_FILE_OPEN 131
#define IDS_HTTPDOWNLOAD_ABORTING_TRANSFER 132
#define IDD_TRANSFER1                   133
#define IDD_FILES                       133
#define IDS_MAIN_READY                  133
#define IDD_SEARCH                      134
#define IDD_SEARCHDLG                   134
#define IDS_MAIN_SOCKETERROR            134
#define IDS_MAIN_BTN_DISCONNECT         135
#define IDS_MAIN_BTN_DISCONNECT_TOOLTIP 136
#define IDS_ERR_WRONGFILEID             137
#define IDS_ERR_BADHASHSET              138
#define IDI_ICON1                       139
#define IDS_ERR_BADDATABLOCK            139
#define IDS_ERR_ILLFORMEDHASH           140
#define IDI_NOTCONNECTED                141
#define IDS_ERR_BADED2KLINK             141
#define IDI_ICON3                       142
#define IDI_CONNECTED                   142
#define IDS_ERR_NOSLLINK                142
#define IDI_UPDOWN                      143
#define IDS_ERR_WRONGPACKAGESIZE        143
#define IDI_UP1DOWN1                    144
#define IDS_ERR_WRONGHPACKAGESIZE       144
#define IDI_UP1DOWN0                    145
#define IDS_ERR_BADCLIENTACTION         145
#define IDI_UP0DOWN1                    146
#define IDS_ERR_UNKNOWNCLIENTACTION     146
#define IDI_USER0                       147
#define IDS_ERR_OPENMET                 147
#define IDI_DCS5                        148
#define IDS_ERR_BADMETVERSION           148
#define IDI_DCS4                        149
#define IDS_ERR_BADGAP                  149
#define IDI_DCS1                        150
#define IDS_ERR_METCORRUPT              150
#define IDI_ICON5                       151
#define IDI_DCS3                        151
#define IDS_ERR_FILEERROR               151
#define IDI_DCS2                        152
#define IDS_ERR_FILEOPEN                152
#define IDD_HTTPDOWNLOAD                153
#define IDS_ERR_REHASH                  153
#define IDD_CHAT                        154
#define IDS_ERR_PART_FNF                154
#define IDS_ERR_OPENMETFILE             155
#define IDR_HTTPDOWNLOAD_ANIMATION      156
#define IDS_ERR_SAVEMET                 156
#define IDI_MESSAGE                     157
#define IDS_ERR_FOUNDCORRUPTION         157
#define IDS_HASHINGDONE                 158
#define IDI_MPENDING                    159
#define IDS_ERR_PARTCORRUPT             159
#define IDI_CHAT                        160
#define IDS_ICHWORKED                   160
#define IDI_CONNECTEDHIGH               161
#define IDS_ERR_OUTOFSPACE              161
#define IDD_SOURCEDETAILWND             162
#define IDS_ERR_WRITEERROR              162
#define IDI_COMPPROT                    163
#define IDS_ERR_COMPLETIONFAILED        163
#define IDI_PLUS                        164
#define IDS_ERR_DELETEFAILED            164
#define IDI_PLUSCOMPROT                 165
#define IDS_DOWNLOADDONE                165
#define IDS_ERR_DELETE                  166
#define IDS_ERR_HASHERRORWARNING        167
#define IDS_ERR_INCOMPLETEHASH          168
#define IDS_DOWNLOADING                 169
#define IDI_BN_CONNECT                  170
#define IDS_WAITING                     170
#define IDI_BN_DISCONNECT               171
#define IDS_HASHING                     171
#define IDI_BN_DOWNLOAD                 172
#define IDS_COMPLETING                  172
#define IDI_BN_FILES                    173
#define IDS_COMPLETE                    173
#define IDS_PAUSED                      174
#define IDI_BN_PREFERENCES              175
#define IDS_ERRORLIKE                   175
#define IDI_BN_SEARCH                   176
#define IDS_ASSIGNED2K                  176
#define IDI_BN_SERVER                   177
#define IDS_ERR_EMPTYADRESSESDAT        177
#define IDI_BN_STATISTICS               178
#define IDS_ERR_EMPTYADRESSESDAT_TITLE  178
#define IDS_ERR_BADSERVERMETVERSION     179
#define IDS_SERVERSFOUND                180
#define IDI_BN_MESSAGES                 181
#define IDS_SERVERSADDED                181
#define IDS_ERR_BADSERVERLIST           182
#define IDS_ERR_FILEERROR_SERVERMET     183
#define IDS_ERR_SAVESERVERMET           184
#define IDR_MAINFRAME5                  185
#define IDR_TRAYICON                    185
#define IDS_MAIN_CONNECTEDTO            185
#define IDS_MAIN_BTN_CANCEL             186
#define IDI_BN_DISCONNECT1              187
#define IDI_BN_STOPCONNECTING           187
#define IDS_AVAIL                       187
#define IDR_TRAYICON1                   188
#define IDR_TRAYICON_GREY               188
#define IDS_TYPE                        188
#define IDS_FILEID                      189
#define IDS_FILE                        190
#define IDS_DOWNLOAD                    191
#define IDS_SL_SERVERNAME               192
#define IDS_IP                          193
#define IDS_DESCRIPTION                 194
#define IDS_PING                        195
#define IDS_FILES                       196
#define IDS_PREFERENCE                  197
#define IDS_PRIONOPREF                  198
#define IDS_CONNECTTHIS                 199
#define IDS_CHAT_WELCOME                200
#define IDS_MAIN_BTN_CANCEL_TOOLTIP     201
#define IDS_MAIN_BTN_CONNECT            202
#define IDS_ERR_SAVESERVERMET2          203
#define IDS_NOTCONNECTED                204
#define IDS_DISCONNECTED                205
#define IDS_USERS                       206
#define IDS_UPDOWN                      207
#define IDS_CONNECTED                   208
#define IDS_LINKNOTADDED                209
#define IDS_MAIN_EXIT                   210
#define IDS_MAIN_EXITTITLE              211
#define IDS_MAIN_POPUP_RESTORE          212
#define IDS_CONNECTTOANYSERVER          213
#define IDS_EXIT                        214
#define IDS_CONNECTING                  215
#define IDS_FAILED                      216
#define IDS_ERR_BADSERVERREPLY          217
#define IDS_NEWCLIENTID                 218
#define IDS_NOSHAREDFOLDERS             219
#define IDS_SHAREDFOLDERS               220
#define IDS_VIEW1                       221
#define IDS_VIEW2                       222
#define IDS_PRIOLOW                     223
#define IDS_PRIONORMAL                  224
#define IDS_PRIOHIGH                    225
#define IDS_DOWNLOADMENUTITLE           226
#define IDS_PRIORITY                    227
#define IDS_NEWSERVERS                  228
#define IDS_DL_STOP                     229
#define IDS_DL_PAUSE                    230
#define IDS_DL_RESUME                   231
#define IDS_DL_OPEN                     232
#define IDS_DL_INFO                     233
#define IDS_DL_CLEAR                    234
#define IDS_DL_LINK1                    235
#define IDS_DL_LINK2                    236
#define IDS_SHOWDETAILS                 237
#define IDS_SEND_MSG                    238
#define IDS_ASKING                      239
#define IDS_CONNVIASERVER               240
#define IDS_ONQUEUE                     241
#define IDS_TRANSFERRING                242
#define IDS_RECHASHSET                  243
#define IDS_NONEEDEDPARTS               244
#define IDS_NOCONNECTLOW2LOW            245
#define IDS_UNKNOWN                     246
#define IDS_ASKED4ANOTHERFILE           247
#define IDS_Q_CANCELDL                  248
#define IDS_Q_CANCELDL2                 249
#define IDS_COPIED2CB                   250
#define IDS_PROCDONE                    251
#define IDS_SELECTINCOMINGDIR           252
#define IDS_SELECTTEMPDIR               253
#define IDS_DL_FILENAME                 254
#define IDS_DL_SIZE                     255
#define IDS_DL_TRANSF                   256
#define IDS_DL_SPEED                    257
#define IDS_DL_PROGRESS                 258
#define IDS_DL_SOURCES                  259
#define IDS_ERR_BADSERVERLISTRECEIVED   260
#define IDS_STATUS                      261
#define IDS_DL_REMAINS                  262
#define IDB_UP                          262
#define IDS_QL_USERNAME                 263
#define IDB_BITMAP2                     263
#define IDB_DOWN                        263
#define IDS_ERR_PACKAGEHANDLING         264
#define IDS_QL_RATING                   265
#define IDS_CONNECTINGTO                266
#define IDS_CLIENTS                     267
#define IDS_SEARCH_ANY                  268
#define IDS_SEARCH_AUDIO                269
#define IDS_SEARCH_PICS                 270
#define IDS_SEARCH_PRG                  271
#define IDS_SEARCH_VIDEO                272
#define IDS_SEARCH_CDIMG                273
#define IDS_SEARCH_ARC                  274
#define IDS_ERR_INVALIDLINK             275
#define IDS_ERR_LINKERROR               276
#define IDS_ERR_NOTCONNECTED            277
#define IDS_REMOVETHIS                  278
#define IDS_REMOVEALL                   279
#define IDS_SRV_ADDR                    280
#define IDS_SRV_PORT                    281
#define IDS_SRV_NOTADDED                282
#define IDS_SRV_READD                   283
#define IDS_SRV_NOURLAV                 284
#define IDS_INVALIDURL                  285
#define IDS_ERR_FAILEDDOWNLOADMET       286
#define IDS_PERMISSION                  287
#define IDS_SHAREDFILES                 288
#define IDS_PRIOVERYLOW                 289
#define IDS_PRIORELEASE                 290
#define IDD_PPG_CONNECTION              290
#define IDS_FSTATUS_LOCKED              291
#define IDD_PPG_DIRECTORIES             291
#define IDS_FSTATUS_FRIENDSONLY         292
#define IDD_PPG_FILES                   292
#define IDS_FSTATUS_PUBLIC              293
#define IDD_PPG_GENERAL                 293
#define IDS_PRIOHIGHEST                 294
#define IDD_PPG_SERVER                  294
#define IDS_ERR_NOPRIMCHANGE            295
#define IDS_HIDDEN                      296
#define IDS_FSTAT_TRANSFER              297
#define IDS_FSTAT_CONNECTION            298
#define IDS_FSTAT_CLIENTS               299
#define IDS_FSTAT_SERVERS               300
#define IDS_FSTAT_WAITING               301
#define IDS_TIME                        302
#define IDS_STATS_DDATA                 303
#define IDS_STATS_UDATA                 304
#define IDS_STATS_AVGDL                 305
#define IDS_STATS_AVGUL                 306
#define IDS_STATS_FOUNDSRC              307
#define IDS_STATS_ACTDL                 308
#define IDS_STATS_ACTUL                 309
#define IDS_STATS_WAITINGUSERS          310
#define IDS_STATS_RECONNECTS            311
#define IDS_STATS_WAITTRANSF            312
#define IDS_STATS_WAITCONN              313
#define IDS_STATS_CONNECTEDSINCE        314
#define IDS_STATS_RATIO                 315
#define IDS_STATS_UNKNOWNCLIENT         316
#define IDS_STATS_TOTAL                 317
#define IDS_STATS_WORKING               318
#define IDS_STATS_DEAD                  319
#define IDS_DISABLED                    320
#define IDS_STOPPED                     321
#define IDS_STATS_UPDATELABEL           322
#define IDS_QUEUECOUNT                  323
#define IDS_NICKNAME                    324
#define IDS_CLIENTSFILENAME             325
#define IDS_ASKEDFAF                    326
#define IDS_WAITED                      327
#define IDS_ERR_LOADCREDITFILE          328
#define IDS_ERR_CREDITFILEOLD           329
#define IDS_CREDITFILELOADED            330
#define IDS_CREDITFILECORRUPT           331
#define IDS_ERR_CREDITFILEREAD          332
#define IDS_ERR_FAILED_CREDITSAVE       333
#define IDS_NOPARTSFOUND                334
#define IDS_FOUNDPARTS                  335
#define IDS_NEWDOWNLOAD                 336
#define IDS_ERR_ALREADY_DOWNLOADING     337
#define IDS_ERR_ALREADY_DOWNLOADED      338
#define IDS_ERR_SERVERMET_BAD           339
#define IDS_ERR_SERVERMET_UNKNOWN       340
#define IDS_ERR_CONNECTIONERROR         341
#define IDS_ERR_SOCKET                  342
#define IDS_SHAREDFOUND                 343
#define IDS_SHAREDFOUNDHASHING          344
#define IDS_ERR_UDPEXT                  345
#define IDS_ERR_INCOMPLETEBLOCK         346
#define IDS_ERR_OPEN                    347
#define IDS_ERR_CLIENTERRORED           348
#define IDS_ERR_REQ_FNF                 349
#define IDS_CLIENTBLOCKED               350
#define IDS_LUSERS                      351
#define IDS_LFILES                      352
#define IDS_DESC                        353
#define IDS_PORT                        354
#define IDS_ERR_NOVALIDSERVERSFOUND     355
#define IDS_CONNECTEDTOREQ              356
#define IDS_ERR_FATAL                   357
#define IDS_ERR_LOSTC                   358
#define IDS_ERR_DEAD                    359
#define IDS_ERR_FULL                    360
#define IDS_ERR_NOSERVERSC              361
#define IDS_CONNECTEDTO                 362
#define IDS_DL_PREVIEW                  363
#define IDS_VIEWQUEUE                   364
#define IDS_VIEWUPLOADS                 365
#define IDS_TW_UPLOADS                  366
#define IDS_TW_DOWNLOADS                367
#define IDS_TW_QUEUE                    368
#define IDS_SW_SEARCHBOX                369
#define IDS_SW_NAME                     370
#define IDS_SW_GLOBAL                   371
#define IDS_SW_TYPE                     372
#define IDS_SW_DIRECTDOWNLOAD           373
#define IDS_SW_LINK                     374
#define IDS_SW_START                    375
#define IDS_SW_CANCEL                   376
#define IDS_SCHEDULER                   376
#define IDS_IRC                         377
#define IDS_SW_RESULT                   378
#define IDS_SW_DOWNLOAD                 379
#define IDS_SV_SERVERLIST               380
#define IDS_SV_SERVERINFO               381
#define IDS_SV_LOG                      382
#define IDS_SV_NEWSERVER                383
#define IDS_SV_ADDRESS                  384
#define IDS_SV_PORT                     385
#define IDS_SV_ADD                      386
#define IDS_SV_MET                      387
#define IDS_SV_UPDATE                   388
#define IDS_SF_FILES                    389
#define IDS_SF_STATISTICS               390
#define IDS_SF_RELOAD                   391
#define IDS_SF_CURRENT                  392
#define IDS_SF_TOTAL                    393
#define IDS_SF_TRANS                    394
#define IDS_SF_ACCEPTED                 395
#define IDS_SF_REQUESTED                396
#define IDS_ST_DOWNLOAD                 397
#define IDS_ST_UPLOAD                   398
#define IDS_ST_CURRENT                  399
#define IDS_ST_SESSION                  400
#define IDS_ST_ACTIVED                  401
#define IDS_ST_ACTIVEU                  402
#define IDS_ST_WAITINGU                 403
#define IDS_CW_MESSAGES                 404
#define IDS_CW_FRIENDS                  405
#define IDS_CW_SEND                     406
#define IDS_CW_CLOSE                    407
#define IDS_PW_GENERAL                  408
#define IDS_PW_DIR                      409
#define IDS_PW_NICK                     410
#define IDS_PW_INCOMING                 411
#define IDS_PW_TEMP                     412
#define IDS_PW_BROWSE                   413
#define IDS_PW_SHARED                   414
#define IDS_PW_CONNECTION               415
#define IDS_PW_DOWNC                    416
#define IDS_PW_DOWNL                    417
#define IDS_PW_UPC                      418
#define IDS_PW_UPL                      419
#define IDS_PW_MAXC                     420
#define IDS_PW_CLIENTPORT               421
#define IDS_PW_AUTOCON                  422
#define IDS_PW_RECON                    423
#define IDS_PW_SERVER                   424
#define IDS_PW_RDEAD                    425
#define IDS_PW_RETRIES                  426
#define IDS_PW_SCORE                    427
#define IDS_PW_USC                      428
#define IDS_PW_UCC                      429
#define IDS_PW_FILTER                   430
#define IDS_PW_USS                      431
#define IDS_PW_MISC                     432
#define IDS_PW_TRAY                     433
#define IDS_PW_BEEP                     434
#define IDS_PW_PROMPT                   435
#define IDS_PW_SPLASH                   436
#define IDS_PW_DBLCLICK                 437
#define IDS_PW_TOOL                     438
#define IDS_PW_FRONT                    439
#define IDS_PW_LANG                     440
#define IDS_PW_FILES                    441
#define IDS_PW_ICH                      442
#define IDS_PW_SHARE                    443
#define IDS_PW_EVER                     444
#define IDS_PW_FRIENDS                  445
#define IDS_PW_NOONE                    446
#define IDS_PW_RESET                    447
#define IDS_PW_APPLY                    448
#define IDS_EM_TRANS                    449
#define IDS_EM_SERVER                   450
#define IDS_EM_SEARCH                   451
#define IDS_EM_FILES                    452
#define IDS_EM_MESSAGES                 453
#define IDS_EM_STATISTIC                454
#define IDS_EM_PREFS                    455
#define IDS_MB_LANGUAGEINFO             456
#define IDS_FD_GENERAL                  457
#define IDS_FD_NAME                     458
#define IDS_FD_MET                      459
#define IDS_FD_HASH                     460
#define IDS_FD_SIZE                     461
#define IDS_FD_STATUS                   462
#define IDS_FD_TRANSFER                 463
#define IDS_FD_SOURCES                  464
#define IDS_FD_PARTS                    465
#define IDS_FD_TRANS                    466
#define IDS_FD_DATARATE                 467
#define IDS_FD_TRANSI                   468
#define IDS_FD_COMPSIZE                 469
#define IDS_FD_ICH			1500
#define IDS_FD_LSTATS1			1510
#define IDS_FD_LSTATS2			1520
#define IDS_FD_LSTATS3			1530
#define IDS_FD_CLOSE                    470
#define IDS_FD_TITLE                    471
#define IDS_CD_GENERAL                  472
#define IDS_CD_UNAME                    473
#define IDS_CD_UHASH                    474
#define IDS_CD_CSOFT                    475
#define IDS_CD_UIP                      476
#define IDS_CD_SIP                      477
#define IDS_CD_VERSION                  478
#define IDS_CD_UID                      479
#define IDS_CD_SNAME                    480
#define IDS_CD_CDOWN                    481
#define IDS_CD_DOWN                     482
#define IDS_CD_ADOWN                    483
#define IDS_CD_TDOWN                    484
#define IDS_CD_UP                       485
#define IDS_CD_AUP                      486
#define IDS_CD_TUP                      487
#define IDS_CD_TRANS                    488
#define IDS_CD_SCORES                   489
#define IDS_CD_MOD                      490
#define IDS_CD_RATING                   491
#define IDS_CD_USCORE                   492
#define IDS_CD_TITLE                    493
#define IDS_EMULEW                      494
#define IDS_EMULEW2                     495
#define IDS_EMULEW3                     496
#define IDS_PW_MAXSOURCES               497
#define IDS_SHAREDFILESCOUNT            498
#define IDS_SF_SIZE                     499
#define IDS_ERR_WRONGPARTNUMBER         500
#define IDS_ERR_BADPORT                 501
#define IDS_ERR_NOHELLO                 502
#define IDS_ERR_INVALIDPACKAGE          503
#define IDS_ERR_CLIENTERROR             504
#define IDS_ERR_BADSIZE                 505
#define IDS_ERR_BADCLIENT2              506
#define IDS_ERR_NOTAFILELINK            507
#define IDS_ERR_CREATEPARTFILE          508
#define IDS_ERR_UDP_MISCONF             509
#define IDS_CHAT_START                  510
#define IDS_FILEPRIO                    511
#define IDS_SCORE                       512
#define IDS_ASKED                       513
#define IDS_LASTSEEN                    514
#define IDS_BANNED                      515
#define IDS_PRIOVERYHIGH                516
#define IDS_UUSERS                      517
#define IDS_UFAILED                     518
#define IDS_OPENFILE                    519
#define IDS_DATE                        520
#define IDS_PARTINFOS                   521
#define IDS_PARTINFOS2                  522
#define IDS_USERID                      523
#define IDS_CLIENT                      524
#define IDS_SERVER                      525
#define IDS_SOURCEINFO                  526
#define IDS_CLIENTSOURCENAME            527
#define IDS_USERINFO                    528
#define IDS_FILESTATS_SESSION           529
#define IDS_FILESTATS_TOTAL             530
#define IDS_REQ_UNKNOWNFILE             531
#define IDS_TRANSFERTIME                532
#define IDS_SF_AVERAGESIZE              533
#define IDS_SF_REQUESTS                 534
#define IDS_SF_ACCEPTS                  535
#define IDS_SF_TRANSFERRED              536
#define IDS_REMOVESEARCHSTRING          537
#define IDS_REMOVESELECTED              538
#define IDS_REMOVEALLSEARCH             539
#define IDS_ENTERQUEUE                  540
#define IDS_SF_FAIL                     541
#define IDS_SF_WUSER                    542
#define IDS_SF_WFILE                    543
#define IDS_SF_USER                     544
#define IDS_SF_FILE                     545
#define IDS_UPLOADTIME                  546
#define IDS_LASTSEENCOMPL               547
#define IDS_YES                         548
#define IDS_NO                          549
#define IDS_UNBAN                       550
#define IDS_SF_WORKING                  551
#define IDS_ADDTOSTATIC                 552
#define IDS_ADDED2SSF                   553
#define IDS_STATICSERVER                554
#define IDS_ERROR_SSF                   555
#define IDS_ERR_PACKAGEERROR            556
#define IDS_SF_ACTIVECON                557
#define IDS_SF_MAXCONLIMITREACHED       558
#define IDS_SF_AVGCON                   559
#define IDS_SF_PEAKCON                  560
#define IDS_ST_ACTIVEC                  561
#define IDS_NETWORK                     562
#define IDS_SF_DELCOUNT                 563
#define IDI_FRIENDS1                    564
#define IDS_QUEUEFULL                   564
#define IDS_AVG                         565
#define IDS_BYTES                       566
#define IDS_KBYTES                      567
#define IDS_MBYTES                      568
#define IDS_GBYTES                      569
#define IDS_TBYTES                      570
#define IDS_SECS                        571
#define IDS_MINS                        572
#define IDS_HOURS                       573
#define IDS_DAYS                        574
#define IDS_ED2KLINKFIX                 575
#define IDS_RECONNECT                   576
#define IDS_ERR_PACKAGEERROR2           577
#define IDS_SERVERADDED                 577
#define IDB_UP2X                        577
#define IDS_STATS_SUCCUPCOUNT           578
#define IDB_DOWN2X                      578
#define IDS_STATS_FAILUPCOUNT           579
#define IDI_SHAREDFILES                 579
#define IDS_STATS_AVEUPTIME             580
#define IDI_SMALLSTATISTICS             580
#define IDS_SEARCHMINSIZE               581
#define IDI_SEARCHRESULTS               581
#define IDS_SEARCHMAXSIZE               582
#define IDI_NORMALSEARCH                582
#define IDS_SEARCHMINSIZE2              583
#define IDS_PREF_ONLINESIG              583
#define IDI_WEBBASED                    583
#define IDS_ERROR_SAVEFILE              584
#define IDI_ICON2                       584
#define IDI_DIRECTDOWNLOAD              584
#define IDS_REQ_SHAREDFILES             585
#define IDI_NEWSERVER                   585
#define IDS_REQUESTED                   586
#define IDS_ACCEPTED                    586
#define IDI_UPDATESERVERS               586
#define IDS_DENIED                      587
#define IDI_SERVERLIST                  587
#define IDI_SERVERINFO                  588
#define IDS_PW_CON_DOWNLBL              588
#define IDI_LOG                         589
#define IDS_PW_CON_UPLBL                589
#define IDI_FRIEND                      590
#define IDS_PW_AUTOCONNECTSTATICONLY    590
#define IDS_PW_CON_LIMITFRM             591
#define IDS_PW_FILE_ICH                 592
#define IDI_UPLOAD                      592
#define IDS_STATS_AVGLABEL              593
#define IDI_USER                        594
#define IDS_GRAPHS                      594
#define IDR_RT_MANIFEST1                595
#define IDS_STREE                       595
#define IDD_STATSSETUP                  596
#define IDS_STATSSETUPINFO              596
#define IDD_PPG_STATS                   596
#define IDS_HARDLIMIT                   597
#define IDS_COL_MORECOLORS              598
#define IDS_KBYTESEC                    599
#define IDS_TOOMANYCONNS                600
#define IDS_DOWNLOADRENAMED             601
#define IDS_TBN_DOWNLOADDONE            602
#define IDS_TBN_DOWNLOADADDED           603
#define IDS_PW_TBN_USESOUND             604
#define IDS_PW_TBN_ONLOG                605
#define IDS_PW_TBN_ONCHAT               606
#define IDS_PW_TBN_POP_ALWAYS           607
#define IDS_PW_TBN_ONDOWNLOAD           608
#define IDS_PW_EKDEV_OPTIONS            609
#define IDS_PW_TASKBARNOTIFIER          610
#define IDS_PW_TBN_OPTIONS              611
#define IDS_TBN_NEWCHATMSG              612
#define IDS_PW_CON_CAPFRM               613
#define IDS_IRC_STATUSLOG               614
#define IDS_IRC_STATUS                  615
#define IDS_IRC_PRIVATECHANSTART        616
#define IDS_IRC_SLAPMSG                 617
#define IDS_IRC_PRIVMESSAGE             618
#define IDS_IRC_OP                      619
#define IDS_IRC_DEOP                    620
#define IDS_IRC_VOICE                   621
#define IDS_IRC_HALFOP                  622
#define IDS_IRC_DEHALFOP                623
#define IDS_IRC_KICK                    624
#define IDS_IRC_SLAP                    625
#define IDS_IRC_JOIN                    626
#define IDS_IRC_CHANNELLIST             627
#define IDS_IRC_CONNECT                 628
#define IDS_IRC_DISCONNECT              629
#define IDS_IRC_HASJOINED               630
#define IDS_IRC_HASPARTED               631
#define IDS_IRC_WASKICKEDBY             632
#define IDS_IRC_NICKUSED                633
#define IDS_IRC_DEVOICE                 634
#define IDS_IRC_HASQUIT                 635
#define IDS_IRC_NOWKNOWNAS              636
#define IDS_IRC_SETSMODE                637
#define IDS_IRC_NOTSUPPORTED            638
#define IDS_IRC_CHANNEL                 639
#define IDS_IRC_NICK                    640
#define IDS_IRC_SEND                    641
#define IDS_IRC_NAME                    642
#define IDS_IRC_SLAPMSGSEND             643
#define IDS_IRC_ADDTIMESTAMP            644
#define IDS_IRC_USEFILTER               645
#define IDS_IRC_PERFORM                 646
#define IDS_IRC_USEPERFORM              647
#define IDS_IRC_ADDTOFRIENDLIST         648
#define IDS_IRC_LOADCHANNELLISTONCON    649
#define IDS_IRC_ADDASFRIEND             650
#define IDS_PW_WARNING                  651
#define IDS_TAKEOVER                    652
#define IDS_CLEANUP                     653
#define IDS_RENAME                      654
#define IDS_IRC_ADDLINKTOIRC            655
#define IDS_IRC_SENDLINK                656
#define IDS_IRC_RECIEVEDLINK            657
#define IDS_IRC_ACCEPTLINKS             658
#define IDR_SPLASH                      658
#define IDS_IRC_ACCEPTLINKS2            659
#define IDR_ABOUT                       659
#define IDS_SOURCENAMES                 659
#define IDS_PREF_STARTMIN               660
#define IDI_BN_IRC                      660
#define IDS_PS_TBN_IMPORTANT            661
#define IDS_COLORS                      662
#define IDI_FRIENDS2                    662
#define IDS_SP_BACKGROUND               663
#define IDI_FRIENDS3                    663
#define IDS_SP_GRID                     664
#define IDS_SP_DL1                      665
#define IDS_SP_DL2                      666
#define IDS_SP_DL3                      667
#define IDS_SP_UL1                      668
#define IDS_SP_UL2                      669
#define IDS_SP_UL3                      670
#define IDS_SP_ACTCON                   671
#define IDS_SP_ACTDL                    672
#define IDS_SP_ACTUL                    673
#define IDS_PW_TIT_UP                   674
#define IDS_PW_TIT_DOWN                 675
#define IDS_PW_UNLIMITED                676
#define IDS_PW_MINREC                   677
#define IDS_PW_UA                       678
#define IDS_PW_PA                       679
#define IDS_IRC_IGNOREINFOMESSAGE       680
#define IDS_CONNECTIONLOST              681
#define IDS_WIZ_OS_FRAME                682
#define IDS_WIZ_CONCURENTDOWN_FRAME     683
#define IDD_FILEINFO                    683
#define IDS_WIZ_CUSTOM_FRAME            684
#define IDS_WIZ_HOTBUTTON_FRAME         685
#define IDS_WIZ_TRUEUPLOAD_TEXT         686
#define IDS_WIZ_TRUEDOWNLOAD_TEXT       687
#define IDS_FRIENDLIST                  688
#define IDS_CHATDISCONNECTED            689
#define IDS_WIZARD                      690
#define IDS_ADDFRIEND                   691
#define IDS_REMOVEFRIEND                692
#define IDS_REMOVEFROMSTOK              693
#define IDS_REMOVEFROMSTATIC            694
#define IDS_LENGTH                      695
#define IDS_VIDEO                       696
#define IDS_AUDIO                       697
#define IDS_FILEINFO                    698
#define IDS_CODEC                       699
#define IDS_BITRATE                     700
#define IDS_WIDTH                       701
#define IDS_FPS                         702
#define IDS_CHANNELS                    703
#define IDS_SAMPLERATE                  704
#define IDS_ROUNDBITRATE                705
#define IDS_HEIGHT                      706
#define IDS_SHAREDFILES_FAILED          707
#define IDS_SHAREDFILES_REQUEST         708
#define IDS_VIEWFILES                   709
#define IDS_FRIENDSLOT                  710
#define IDS_FRIENDDETAIL                711
#define IDI_PREF_SERVER                 711
#define IDI_PREF_NOTIFICATIONS          712
#define IDS_ASCWOP_PROXYSUPPORT         712
#define IDS_UDPDISABLED                 713
#define IDI_PREF_CONNECTION             713
#define IDI_PREF_FILES                  714
#define IDS_CTINFO                      714
#define IDI_PREF_FOLDERS                715
#define IDS_WIZ_CTFRAME                 715
#define IDI_PREF_GENERAL                716
#define IDS_CONCURDWL                   716
#define IDI_PREF_IRC                    717
#define IDS_UNIT                        717
#define IDI_PREF_STATISTICS             718
#define IDS_WIZARD_CUSTOM               718
#define IDS_WIZARD_ENTERBELOW           719
#define IDI_PREF_TWEAK                  719
#define IDS_STATS_MAXAVGDL              720
#define IDD_COMMENT                     720
#define IDS_STATS_MAXDL                 721
#define IDD_COMMENTLST                  721
#define IDS_PW_FASTSRVCON               722
#define IDS_SOURCELINKFAILED            723
#define IDI_CLOSE                       723
#define IDS_CREATESOURCELINK            724
#define IDI_MLDONK                      724
#define IDS_ERR_LOADSERVERMET           725
#define IDI_PLUSMLDONK                  725
#define IDS_ERR_CORRUPTCOMPRPKG         726
#define IDS_ERR_EMFRIENDSINVALID        727
#define IDS_ERR_READEMFRIENDS           728
#define IDS_ERR_KNOWNSERVERINFOREC      729
#define IDS_ERR_TIMERCREATEFAILED       730
#define IDS_WIZ_DOWN                    731
#define IDS_WIZ_UP                      732
#define IDS_SELECT_INCOMINGDIR          733
#define IDS_SELECT_TEMPDIR              734
#define IDS_PRIOAUTO                    735
#define IDS_PRIOAUTONORMAL              736
#define IDS_PRIOAUTOHIGH                737
#define IDS_PRIOAUTORELEASE             738
#define IDS_PRIOAUTOLOW                 739
#define IDS_ERR_CONTIMEOUT              740
#define IDS_ERR_CONFAILED               741
#define IDS_OUTOFSERVERS                742
#define IDS_PW_CONLIMITS                743
#define IDS_MAXCON5SECLABEL             744
#define IDS_PW_UAP                      745
#define IDS_MAXCLIENT                   746
#define IDS_PW_TWEAK                    747
#define IDS_WARNING                     748
#define IDS_ADDAFRIEND                  749
#define IDS_PAF_REQINFO                 750
#define IDS_PAF_MOREINFO                751
#define IDS_ERR_NOVALIDFRIENDINFO       752
#define IDS_ADD                         753
#define IDS_RECOVERED_PARTMET           754
#define IDS_CMT_QUEST                   755
#define IDS_CMT_AIDE                    756
#define IDS_CMT_TITLE                   757
#define IDS_CMT_ADD                     758
#define IDS_CMT_NONE                    759
#define IDS_CMT_READ                    760
#define IDS_CMT_NOTCONNECTED            761
#define IDS_CMT_SHOWALL                 762
#define IDS_CMT_REFRESH                 763
#define IDS_CMT_READALL                 764
#define IDS_CMT_NOCOMMENT               765
#define IDS_CMT_RATEQUEST               766
#define IDS_CMT_RATEHELP                767
#define IDS_CMT_FAKE                    768
#define IDS_CMT_POOR                    769
#define IDS_CMT_GOOD                    770
#define IDS_CMT_FAIR                    771
#define IDS_CMT_EXCELLENT               772
#define IDS_CMT_NOTRATED                773
#define IDS_KILO                        774
#define IDS_MEGA                        775
#define IDS_GIGA                        776
#define IDS_TERRA                       777
#define IDS_IRC_EMULEPROTO_IGNOREINFOMESSAGE 778
#define IDS_SMARTLOWIDCHECK             779
#define IDS_SF_SRVOCC                   780
#define IDS_3DDEP                       781
#define IDS_SAMEUSERHASH2               782
#define IDS_AUTOTAKEED2KLINKS           782
#define IDS_ADDNEWFILESPAUSED           783
#define IDS_DOWNLOADMOVIECHUNKS         784
#define IDS_UPDATEQUEUE                 785
#define IDS_RECOVER_SINGLE              786
#define IDS_RECOVER_MULTIPLE            787
#define IDS_ATTEMPTING_RECOVERY         788
#define IDS_RECOVERY_SUCCESSFUL         789
#define IDS_RECOVERY_FAILED             790
#define IDS_STARTNEXTFILE               791
#define IDS_FULLCHUNKTRANS              792
#define IDS_MANUALSERVERHIGHPRIO        793
#define IDS_VERBOSE                     794
#define IDS_FLAT                        795
#define IDS_ROUND                       796
#define IDS_SAMEUSERHASH                797
#define IDS_SEARCHEXTENTION             798
#define IDI_RATING                      798
#define IDS_SEARCHAVAIL                 799
#define IDI_RATINGBAD                   799
#define IDS_ERR_NOHELP                  801
#define IDS_SHOWOVERHEAD                802
#define IDS_UPDOWNSMALL                 803
#define IDS_UPDOWN2                     803
#define IDS_EDITLIST                    804
#define IDS_SP_ICONBAR                  805
#define IDS_RATINGRECV                  806
#define IDS_STRING807                   807
#define IDS_DESCRIPTIONRECV             807
#define IDS_WEBSERVICES                 808
#define IDS_WEBSVEDIT                   809
//#define IDC_CHECK4UPDATE                810
#define IDS_CHECK4UPDATE                810
#define IDI_MYINFO                      810
#define IDS_STARTUP                     811
#define IDI_EDONKEYHYBRID               811
#define IDS_NEWVERSIONAVL               812
#define IDI_PLUSEDONKEYHYBRID           812
#define IDS_NEWVERSIONAVLPOPUP          813
#define IDS_NEWVERSIONFAILED            814
#define IDS_CB_TBN_ONNEWVERSION         815
#define IDS_NONEWERVERSION              816
#define IDS_VERSIONCHECK                817
#define IDS_VISITVERSIONCHECK           818
#define IDS_PW_VIDEOPLAYER              819
#define IDS_VIDEOBACKUP                 820
#define IDS_STATIC_EMPTY                821
#define IDS_MYINFO                      822
#define IDS_ID                          823
#define IDS_IDLOW                       824
#define IDS_IDHIGH                      825
#define IDS_UPSTATUS                    826
#define IDS_TOVERHEAD                   827
#define IDS_FROVERHEAD                  828
#define IDS_SSOVERHEAD                  829
#define IDS_SOVERHEAD                   830
#define IDS_ERR_FILECOMPLETIONTHREAD    831
#define IDS_FILEBUFFERSIZE              832
#define IDS_QUEUESIZE                   833
#define IDS_DAYS2                       834
#define IDS_ERR_MAKEBAKCREDITFILE       835
#define IDS_COMMENT                     836
#define IDS_IPFILTERED                  837
#define IDS_DL_TRANSFCOMPL              838
#define IDS_STATS_FILTEREDCLIENTS       839
#define IDS_FILTERLABEL                 840
#define IDS_ERR_LARGEREQBLOCK           841
#define IDS_SHOWRATEONTITLE             842
#define IDS_ERR_TOMANYCONS              843
#define IDS_TBN_NOMESSAGEHISTORY        843
#define IDS_PW_DAP                      844
#define IDS_FD_LASTCHANGE               845
#define IDS_STATS_RUNTIME               846
#define IDS_ALL                         847
#define IDS_CAT_ADD                     848
#define IDS_CAT_EDIT                    849
#define IDS_CAT_REMOVE                  850
#define IDS_CAT                         851
#define IDS_TITLE                       852
#define IDS_COLOR                       853
#define IDS_DEFAULT                     854
#define IDS_STRING854                   854
#define IDS_PW_DISPLAY                  855
#define IDS_CAT_INVADD                  856
#define IDS_CAT_REINVADD                857
#define IDS_INDICATERATINGS             858
#define IDS_ALLOTHERS                   859
#define IDS_TOCAT                       860
#define IDS_CAT_UNASSIGN                861
#define IDS_ADDDOWNLOADSFROMCB          862
#define IDS_ADDDOWNLOADSFROMCBTITLE     863
#define IDS_PF_WATCHCB                  864
#define IDS_SECURITY                    865
#define IDS_IPFILTERLOADED              866
#define IDS_IPFILTER                    867
#define IDS_EDIT                        868
#define IDS_FILTERLEVEL                 869
#define IDS_FILTERSERVERBYIPFILTER      870
#define IDS_FILTERCOMMENTSLABEL         871
#define IDS_CLIENTLIST                  872
#define IDS_FOLDER                      873
#define IDS_CL_UPLOADSTATUS             874
#define IDS_CL_TRANSFUP                 875
#define IDS_CL_DOWNLSTATUS              876
#define IDS_CL_TRANSFDOWN               877
#define IDS_CL_PENDING                  878
#define IDS_CL_LOW2LOW                  879
#define IDS_ERROR                       880
#define IDS_OPENINC                     881
#define IDS_HOTMENU                     882
#define IDS_DISABLEKNOWNLIST            883
#define IDS_DISABLEQUEUELIST            884
#define IDS_USECREDITSYSTEM             885
#define IDS_FIRSTAUTODOWN               886
#define IDS_FIRSTAUTOUP                 887
#define IDS_FIRSTCREDITSYSTEM           888
#define IDS_FIRSTFULLCHUNK              889
#define IDS_FIRSTSAFECON                890
#define IDS_AUTOCON                     891
#define IDS_FIRSTRUNASSIST              891
#define IDS_FIRSTAUTOCON                892
#define IDS_FIRSTCOMPLETE               893
#define IDS_ENTERUSERNAME               894
#define IDS_STATIC_CPUMEM               895
#define IDS_LOG2DISKFRAME               896
#define IDS_LOG2DISK                    897
#define IDS_DEBUG2DISK                  898
#define IDD_INPUTBOX                    899
#define IDS_EDITCAT                     899
#define IDS_PW_WS                       900
#define IDS_ENABLED                     901
#define IDS_WEB_LOGIN                   902
#define IDS_WEB_ENTER_PASSWORD          903
#define IDS_WEB_LOGIN_NOW               904
#define IDS_WEB_CONFIRM_REMOVE_SERVER   905
#define IDS_WEB_CONTROL                 906
#define IDS_WEB_LOGOUT                  907
#define IDS_WEB_ACTIONS                 908
#define IDS_WEB_SHOW_UPLOAD_QUEUE       909
#define IDS_WEB_HIDE_UPLOAD_QUEUE       910
#define IDS_WEB_GZIP_COMPRESSION        911
#define IDS_WEB_GZIP_COMMENT            912
#define IDS_WEB_UPLOAD_QUEUE_COMMENT    913
#define IDS_WEB_REFRESH_TIME            914
#define IDS_WEB_REFRESH_COMMENT         915
#define IDS_SV_URL                      916
#define IDS_WS_RELOAD_TMPL              917
#define IDS_SV_DEBUGLOG                 918
#define IDS_WS_PASS                     919
#define IDS_WEB_FILESETTINGS            920
#define IDS_WS_ERR_LOADTEMPLATE         921
#define IDS_WEB_CONLIMITS               922
#define IDS_INFLST_USER_TOTALDOWNLOAD   923
#define IDS_INFLST_USER_TOTALUPLOAD     924
#define IDS_PRIORITY_UP                 925
#define IDS_PRIORITY_DOWN               926
#define IDS_SPEED_LIMITS                927
#define IDS_CAPACITY_LIMITS             928
#define IDS_ADMIN                       929
#define IDS_ACCESSDENIED                930
#define IDS_WEB_LOWUSER                 931
#define IDS_STRING932                   932
#define IDS_WEB_ERR_CANTLOAD            932
#define IDS_GLOBALSEARCH                933
#define IDS_METHOD                      934
#define IDS_SW_WEBBASED                 935
#define IDS_NOTHING                     936
#define IDS_SETUPSPEED                  937
#define IDI_PREF_SCHEDULER              937
#define IDS_SETDOWNSPEED                938
#define IDD_PPG_SCHEDULER               938
#define IDS_DAYLY                       939
#define IDS_DAY_MO_FR                   940
#define IDS_DAY_SA_SO                   941
#define IDS_MO                          942
#define IDS_DI                          943
#define IDS_MI                          944
#define IDS_DO                          945
#define IDS_FR                          946
#define IDS_SA                          947
#define IDS_SO                          948
#define IDS_WEB_ADMINLOGIN              949
#define IDS_WEB_GUESTLOGIN              950
#define IDS_WEB_SESSIONEND              951
#define IDS_DETAILS                     952
#define IDS_ACTION                      953
#define IDS_REMOVE                      954
#define IDS_NEW                         955
#define IDS_WEB_BADLOGINATTEMPT         956
#define IDS_ERR_TOOLARGEFILE            957
#define IDS_MSGONLYFRIENDS              958
#define IDS_MSGONLYSEC                  959
#define IDS_MO_SA                       960
#define IDS_DAY_MO_SA                   960
#define IDS_SCHEDULER2                  960
#define IDS_LIMITSOURCES                961
#define IDS_LIMITCONS5SEC               962
#define IDS_SCHED_CATSTOP               963
#define IDS_SCHED_CATRESUME             964
#define IDS_S_DAYS                      965
#define IDS_STARTTIME                   966
#define IDS_VALUE                       967
#define IDS_SCHED_ENTERDATARATELIMIT    968
#define IDS_SCHED_ENTERVAL              969
#define IDS_SCHED_ACTCONFIG             970
#define IDS_SELECTCAT                   971
#define IDS_ALLUNASSIGNED               972
#define IDS_ACTSESSIONS                 973
#define IDS_WEBSRV                      974
#define IDS_SW_REFETCHRES               975
#define IDS_FILEHASH                    976
#define IDS_GLOBALSEARCH2               977
#define IDS_PW_PROXY                    978
#define IDS_PROXY_ENABLE                979
#define IDS_PROXY_TYPE                  980
#define IDS_PROXY_HOST                  981
#define IDS_PROXY_PORT                  982
#define IDS_PROXY_AUTH                  983
#define IDS_STRING984                   984
#define IDS_AUTH                        984
#define IDS_SHOWCATINFO                 985
#define IDS_SW_SEARCHINGINFO            986
#define IDS_LOGENTRY                    987
#define IDS_COPY                        988
#define IDS_SELECTALL                   989
#define IDS_AUTOSCROLL                  990
#define IDS_WIZ1_WELCOME_TITLE          991
#define IDS_WIZ1_WELCOME_ACTIONS        992
#define IDS_WIZ1_WELCOME_BTN_HINT       993
#define IDS_WIZ1_END_TITLE              994
#define IDS_WIZ1                        995
#define IDS_WIZ1_END_BTN_HINT           996
#define IDS_WIZ1_UPLOAD_SUBTITLE        997
#define IDS_DL_RESUMENEXT               998
#define IDS_DONTCHANGE                  999
#define IDS_STARTPRIO                   1000
#define IDS_PREF_STARTNEXTFILECAT       1001
#define IDS_REPAINTGRAPHS               1002
#define IDS_SOFTFILES                   1003
#define IDS_HARDFILES                   1004
#define IDS_FILTER                      1005
#define IDS_CHANGECATVIEW               1025
#define IDS_STATUS_NOTCOMPLETED         1026
#define IDS_POWERSHARE                  1027
#define IDC_ANIMATE1                    2000
#define IDC_STATUS                      2001
#define IDC_FILESTATUS                  2002
#define IDC_TIMELEFT                    2004
#define IDC_TRANSFER_RATE               2005
//#define IDC_MAXSOURCEPERFILE            2006
//#define IDC_EDITSEARCHMIN               2006
#define IDC_INPUTWINDOW                 2006
#define IDC_IRC_SERVER_BOX              2006
//#define IDC_MAXDOWN                     2007
#define IDC_ELINK                       2007
#define IDC_IRC_NAME_BOX                2007
//#define IDC_NICK                        2008
#define IDC_LOGBOX                      2008
//#define IDC_EDITSEARCHMAX               2008
#define IDC_TITLEWINDOW                 2008
#define IDC_IRC_NICK_BOX                2008
#define IDC_TRUEDOWNLOAD_BOX            2008
#define IDC_WIZ_TRUEDOWNLOAD_BOX        2008
#define IDC_MAXSRCEPERFILEUDP           2008
//#define IDC_VIDEOPLAYER                 2008
#define IDC_BUTTON2                     2009
#define IDC_EDIT7                       2009
//#define IDC_MAXCON                      2009
//#define IDC_RENAME                      2009
//#define IDC_EDITSEARCHEXTENSION         2009
#define IDC_QUEUE_REFRESH_BUTTON        2009
//#define IDC_BROWSEV                     2009
#define IDC_REMOVE                      2009
#define IDC_TMPLBROWSE                  2009
#define IDC_LIST1                       2010
#define IDC_EDIT8                       2010
//#define IDC_INCFILES                    2010
#define IDC_UPLOADLIST                  2010
#define IDC_NICKLIST                    2010
#define IDC_IRC_PERFORM_BOX             2010
//#define IDC_MAXCON5SEC                  2010
//#define IDC_EDITSEARCHAVAIBILITY        2010
#define IDC_MYINFOLIST                  2010
#define IDC_BUTTON1                     2011
#define IDC_LIST2                       2011
#define IDC_DOWNLOADLIST                2011
#define IDC_SBUTTON1                    2011
//#define IDC_RELOADSHAREDFILES           2011
#define IDC_BN_DOWNLOADS                2011
#define IDC_MAXCON2                     2011
//#define IDC_TOOLTIPDELAY                2011
//#define IDC_CLEARALL                    2011
//#define IDC_SELINCDIR                   2011
//#define IDC_TAKEOVER                    2011
#define IDC_128K_BUTTON                 2011
#define IDC_WIZ_56K_BUTTON              2011
#define IDC_ADD                         2011
//#define IDC_COLOR_BUTTON                2011
//#define IDC_EDITADR                     2011
//#define IDC_BROWSE                      2011
#define IDC_RELOADFILTER                2011
#define IDC_HOTMENU                     2011
#define IDC_WSRELOADTMPL                2011
#define IDC_TRAFFIC_TEXT                2012
#define IDC_BUTTON3                     2012
#define IDC_CLOSECHAT                   2012
#define IDC_BN_IRC                      2012
#define IDC_512K_BUTTON                 2012
#define IDC_WIZ_256K_BUTTON             2012
//#define IDC_CMTBT                       2012
//#define IDC_SEARCH_RESET                2012
#define IDC_COLOR_BUTTON2               2012
#define IDC_SERVINF_TEXT                2013
#define IDC_TRAFFIC_TEXT2               2013
#define IDC_UPLOAD_TEXT                 2013
#define IDC_SERVLIST_TEXT               2014
#define IDC_TRAFFIC_TEXT3               2014
#define IDC_LOG_TEXT                    2015
#define IDC_TRAFFIC_TEXT4               2015
#define IDC_TRAFFIC_TEXT5               2016
#define IDC_TREE1                       2017
#define IDC_SERVLIST                    2017
#define IDC_TRAFFIC_TEXT10              2017
//#define IDC_CANCELS                     2017
#define IDC_STATTREE                    2017
#define IDC_IPADDRESS1                  2018
#define IDC_TRAFFIC_TEXT6               2018
//#define IDC_IPADDRESS                   2018
//#define IDC_SDOWNLOAD                   2018
#define IDC_TRAFFIC_TEXT7               2019
#define IDC_FAKECHECK                   2019
#define IDC_EDIT3                       2020
#define IDC_TRAFFIC_TEXT8               2020
//#define IDC_SPORT                       2020
//#define IDC_SERVERRETRIES               2020
#define IDC_ASKFRIENDIP                 2020
#define IDC_STATUSWINDOW                2020
#define IDC_IRC_MINUSER_BOX             2020
#define IDC_CHECK1                      2021
#define IDC_SHOWCATINFO                 2021
#define IDC_SERVERMETURL                2022
#define IDC_MARKDEADSERVER              2022
#define IDC_SWEB                        2022
#define IDC_RADIO_US_98ME               2022
#define IDC_WIZ_XP_RADIO                2022
//#define IDC_SHOWRATEONTITLE             2022
#define IDC_DEBUG2DISK                  2022
#define IDC_WSENABLEDLOW                2022
#define IDC_S_ENABLE                    2022
#define IDC_CHECK3                      2023
//#define IDC_SCORE                       2023
#define IDC_ASKFRIENDPORT               2023
#define IDC_IRC_USEPERFORM              2023
//#define IDC_FULLCHUNKTRANS              2023
//#define IDC_SHOWOVERHEAD                2023
#define IDC_INDICATERATINGS             2023
#define IDC_RADIO2                      2024
#define IDC_RADIO_OS_NTXP               2024
#define IDC_WIZ_ME_RADIO                2024
#define IDC_TRAFFIC_TEXT9               2025
#define IDC_TRAFFIC_TEXT11              2026
#define IDC_EDIT9                       2027
//#define IDC_TEMPFILES                   2027
#define IDC_TRAFFIC_TEXT12              2028
#define IDC_BUTTON5                     2029
#define IDC_BROWSE1                     2029
#define IDC_BN_FILES                    2029
#define IDC_RESET                       2029
#define IDC_WIZ_APPLY_BUTTON            2029
#define IDC_BUTTON6                     2030
#define IDC_BROWSE2                     2030
#define IDC_BN_PREFERENCES              2030
#define IDC_WIZ_CANCEL_BUTTON           2030
#define IDC_LIST4                       2031
#define IDC_SDLIST                      2031
#define IDC_BUTTON7                     2032
#define IDC_BN_MESSAGES                 2032
#define IDC_CANCEL                      2032
#define IDC_BUTTON8                     2033
#define IDC_BN_STATISTICS               2033
#define IDC_TRAFFIC_TEXT13              2034
//#define IDC_PORT                        2035
//#define IDC_MAXUP                       2036
#define IDC_APPLY                       2037
#define IDC_TRAFFIC_TEXT14              2038
//#define IDC_UPLOAD_CAP                  2038
#define IDC_SFLIST                      2039
//#define IDC_ICH                         2039
#define IDC_ADDSERVER                   2040
#define IDC_TRAFFIC_TEXT15              2040
//#define IDC_DOWNLOAD_CAP                2040
#define IDC_SEARCHLIST                  2041
#define IDC_NOTIFY                      2041
#define IDC_UPDATESERVERMETFROMURL      2041
#define IDC_COMBO1                      2042
//#define IDC_AUTOSERVER                  2042
#define IDC_Type                        2042
#define IDC_LENGTH                      2042
#define IDC_PROGRESS1                   2043
//#define IDC_MINTRAY                     2043
#define IDC_VCODEC                      2043
//#define IDC_SEARCHNAME                  2044
#define IDC_RECONN2                     2044
//#define IDC_AUTOCONNECT                 2044
#define IDC_VBITRATE                    2044
//#define IDC_STARTS                      2045
//#define IDC_UPDATESERVERCONNECT         2045
#define IDC_VWIDTH                      2045
#define IDC_QUEUECOUNT                  2046
#define IDC_UPDATESERVERCLIENTS         2046
//#define IDC_UPDATESERVERCLIENT          2046
#define IDC_VHEIGHT                     2046
//#define IDC_FLAT                        2047
#define IDC_VFPS                        2047
#define IDC_ACODEC                      2048
#define IDC_ABITRATE                    2049
//#define IDC_CSEND                       2050
#define IDC_ACHANNEL                    2050
#define IDC_ASAMPLERATE                 2051
//#define IDC_CCLOSE                      2055
#define IDC_CHATSEL                     2060
//#define IDC_CMESSAGE                    2061
#define IDC_SSTATIC2                    2062
#define IDC_SSTATIC                     2063
#define IDC_SSTATIC3                    2064
#define IDC_SSTATIC4                    2065
#define IDC_SSTATIC5                    2066
#define IDC_TSTATIC1                    2067
#define IDC_SSTATIC6                    2067
#define IDC_FSTATIC1                    2068
#define IDC_SSTATIC7                    2068
#define IDC_CURSESSION_LBL              2068
#define IIDC_FSTATIC2                   2069
#define IDC_SCURS                       2069
#define IDC_MYINFO                      2069
#define IDC_FSTATIC3                    2070
//#define IDC_MSTATIC3                    2070
#define IDC_FSTATIC4                    2071
#define IDC_FSTATIC5                    2072
#define IDC_CSTATIC1                    2073
#define IDC_FSTATIC6                    2073
#define IDC_FRIENDS_LBL                 2073
#define IDC_DNAME                       2074
#define IDC_FSTATIC7                    2074
#define IDC_DSOFT                       2075
//#define IDC_SREQUESTED                  2075
#define IDC_FSTATIC10                   2075
#define IDC_NAME21                      2075
#define IDC_DIP                         2076
//#define IDC_SACCEPTED                   2076
#define IDC_DSIP                        2077
//#define IDC_STRANSFERED                 2077
#define IDC_DHASH                       2078
#define IDC_FSTATIC8                    2078
#define IDC_DVERSION                    2079
#define IDC_FSTATIC9                    2079
#define IDC_DID                         2080
//#define IDC_SREQUESTED2                 2080
#define IDC_DSNAME                      2081
//#define IDC_SACCEPTED2                  2081
#define IDC_DDOWNLOADING                2082
//#define IDC_STRANSFERED2                2082
#define IDC_DDOWN                       2083
#define IDC_DAVDR                       2084
#define IDC_DDUP                        2085
#define IDC_DAVUR                       2086
#define IDC_DDOWNTOTAL                  2087
#define IDC_DUPTOTAL                    2088
#define IDC_DRATIO                      2089
#define IDC_DRATING                     2092
#define IDC_DSCORE                      2093
#define IDC_GENERAL                     2094
#define IDC_DTRANS                      2095
#define IDC_FD_XI1                      2095
#define IDC_DSCOREM                     2096
#define IDC_FD_XI2                      2096
#define IDC_POPBAR                      2097
#define IDC_SALLTIME                    2098
#define IDC_FD_XI5                      2098
#define IDC_POPBAR2                     2099
//#define IDC_SHARESELECTOR               2099
#define IDC_FD_XI7                      2099
#define IDC_POPBAR3                     2100
#define IDC_SCOPE                       2100
#define IDC_SCOPE_D                     2100
#define IDC_FD_XI9                      2100
#define IDC_TRAFFIC_TEXT1               2101
#define IDC_UPLOAD                      2101
#define IDC_FD_XI11                     2101
#define IDC_BN_SERVERS                  2102
//#define IDC_SLIDER                      2102
#define IDC_FD_XI13                     2102
#define IDC_BN_SEARCH                   2103
//#define IDC_C1                          2103
#define IDC_FD_XI6                      2103
#define IDC_STATIC_LINE                 2104
#define IDC_FD_XI8                      2104
#define IDC_SLIDER_VALUE                2105
#define IDC_FD_XI10                     2105
//#define IDC_C0                          2106
#define IDC_FD_XI12                     2106
#define IDC_STATIC_D                    2107
#define IDC_STATIC_U                    2108
#define IDC_STATISTICS                  2109
#define IDC_STATS                       2110
#define IDC_STATSSCOPE                  2110
//#define IDC_S0                          2111
//#define IDC_S1                          2112
//#define IDC_S3                          2113
//#define IDC_S2                          2114
#define IDD_STATISTICS                  2115
#define IDC_STATIC7                     2115
//#define IDC_C0_2                        2115
#define IDC_STATIC_S0                   2116
#define IDD_SPLASH                      2116
#define IDC_STATIC_S1                   2117
#define IDD_FILEDETAILS                 2117
#define IDC_STATIC_S2                   2118
#define IDD_IRC                         2118
#define IDC_STATIC_S3                   2119
#define IDC_SCOPE_U                     2120
#define IDC_STATIC_D2                   2121
#define IDC_STATIC_D3                   2122
//#define IDC_BEEPER                      2123
#define IDC_STATIC_U2                   2123
#define IDC_STATIC_U3                   2124
//#define IDC_EXIT                        2124
//#define IDC_C2                          2125
//#define IDC_C1_2                        2125
#define IDC_SPIN1                       2125
//#define IDC_SPLASHON                    2125
#define IDC_FRAME2                      2126
#define IDC_PUBLICSHARED                2126
//#define IDC_C0_3                        2126
#define IDC_FRAME1                      2127
#define IDC_STATIC_D4                   2127
#define IDC_TIMEAVG1                    2127
//#define IDC_TypeSearch                  2128
#define IDC_VERSIONTEXT                 2130
#define IDC_STATIC_U4                   2130
#define IDC_TIMEAVG2                    2130
#define IDC_CHECK2                      2131
//#define IDC_FILTER                      2131
//#define IDC_C1_3                        2131
#define IDC_IRC_USECHANFILTER           2131
//#define IDC_UDPDISABLE                  2131
//#define IDC_VERBOSE                     2131
//#define IDC_PREVIEWPRIO                 2131
#define IDC_DISABLEKNOWNLIST            2131
//#define IDC_FNAME                       2132
#define IDC_COMMENTFILTER               2132
//#define IDC_METFILE                     2133
//#define IDC_ADDNEWFILESPAUSED           2133
//#define IDC_FHASH                       2134
//#define IDC_FSIZE                       2135
//#define IDC_SAFESERVERCONNECT           2136
//#define IDC_AUTOTAKEED2KLINKS           2137
//#define IDC_PFSTATUS                    2138
//#define IDC_PARTCOUNT                   2139
//#define IDC_TRANSFERED                  2140
//#define IDC_COMPLSIZE                   2141
//#define IDC_PROCCOMPL                   2142
//#define IDC_DATARATE                    2143
//#define IDC_SOURCECOUNT                 2144
//#define IDC_SOURCECOUNT2                2145
#define IDC_SPLITTER                    2146
//#define IDC_PARTAVAILABLE               2147
//#define IDC_DBLCLICK                    2148
//#define IDC_LASTSEENCOMPL               2149
//#define IDC_STARTMIN                    2150
#define IDC_LASTRECEIVED                2151
//#define IDC_LANGS                       2152
//#define IDC_SEESHARE1                   2154
//#define IDC_SEESHARE2                   2155
#define IDC_CHECK4                      2156
//#define IDC_BRINGTOFOREGROUND           2157
#define IDC_IRC_LISTONCONNECT           2158
//#define IDC_AUTOCONNECTSTATICONLY       2159
//#define IDC_STARTNEXTFILE               2160
#define IDC_DISABLEQUEUELIST            2161
//#define IDC_SEESHARE3                   2162
//#define IDC_REMOVEDEAD                  2163
#define IDC_QUEUELIST                   2164
#define IDC_SERVERCHANNELLIST           2165
//#define IDC_LISTCTRLFILENAMES           2166
#define IDC_S_TITLE                     2167
#define IDC_IRC_MISC_FRM                2168
#define IDC_UDPPORTLABEL                2169
//#define IDC_CMSTATUS                    2170
#define IDC_LANG                        2171
#define IDC_LANG_FRM                    2172
#define IDC_CONNTITLE                   2173
//#define IDC_FD_X0                       2174
//#define IDC_FD_X1                       2175
//#define IDC_FD_X5                       2176
//#define IDC_FD_X6                       2177
//#define IDC_FD_X7                       2178
#define IDC_FD_X8                       2179
//#define IDC_FD_X9                       2180
#define IDC_STATIC30                    2180
//#define IDC_FD_X10                      2181
#define IDC_STATIC31                    2181
//#define IDC_FD_X11                      2182
#define IDC_STATIC32                    2182
//#define IDC_FD_X12                      2183
#define IDC_STATIC33                    2183
//#define IDC_FD_X13                      2184
#define IDC_STATIC34                    2184
//#define IDC_FD_X14                      2185
#define IDC_STATIC35                    2185
#define IDC_STATIC36                    2186
//#define IDC_FD_X15                      2186
//#define IDC_FD_STATS                    2187
#define IDC_STATIC38                    2188
#define IDC_FD_LASTCHANGE               2188
#define IDC_STATIC40                    2189
#define IDC_STATIC41                    2190
#define IDC_STATIC42                    2191
#define IDC_STATIC43                    2192
#define IDC_STATIC44                    2193
#define IDC_STATIC45                    2194
#define IDC_STATIC46                    2195
#define IDC_STATIC47                    2196
#define IDC_STATIC50                    2197
#define IDC_STATIC51                    2198
#define IDC_STATIC52                    2199
#define IDC_STATIC53                    2200
#define IDC_STATICFIX1                  2201
#define IDC_TASKBARNOTIFIER             2201
#define IDC_STATICFIX2                  2202
//#define IDC_RESULTS_LBL                 2202
#define IDC_MSTATIC5                    2203
//#define IDC_SEARCH_FRM                  2203
#define IDC_MSTATIC1                    2204
#define IDC_WEBSEARCH_FRM               2204
#define IDC_MSTATIC6                    2205
#define IDC_DDOWN_FRM                   2205
//#define IDC_MSTATIC4                    2206
#define IDC_MSTATIC7                    2207
#define IDC_BUTTON4                     2208
//#define IDC_SEARCHMINSIZE               2208
#define IDC_FRIENDSICON                 2208
#define IDC_CHATSEND                    2208
#define IDC_56K_BUTTON                  2208
#define IDC_WIZ_512K_BUTTON             2208
#define IDC_ED2KFIX                     2209
//#define IDC_SEARCHMAXSIZE               2209
#define IDC_MESSAGEICON                 2209
#define IDC_LOGRESET                    2210
#define IDC_TAB1                        2210
#define IDC_TOTAL_LBL                   2210
#define IDC_WEBSVEDIT                   2210
#define IDC_SERVMSG                     2211
//#define IDC_ONLINESIG                   2211
#define IDC_SEARCHLST_ICO               2211
#define IDC_SERVLST_ICO                 2212
//#define IDC_SEARCHEXTENTION             2212
#define IDC_SERVINF_ICO                 2213
#define IDC_DOWNLOAD_ICO                2213
//#define IDC_SEARCHAVAIL                 2213
#define IDC_LOG_ICO                     2214
#define IDC_DOWNLOAD_TEXT               2214
#define IDC_METH                        2214
#define IDC_UPLOAD_ICO                  2215
#define IDC_CHANNELLIST                 2216
#define IDC_BN_IRCCONNECT               2217
#define IDC_RICHEDIT21                  2218
#define IDC_TBN_OPTIONS                 2218
//#define IDC_SLIDERINFO                  2220
//#define IDC_SLIDERINFO2                 2221
//#define IDC_SLIDER2                     2223
//#define IDC_SLIDER3                     2224
//#define IDC_COLORSELECTOR               2225
#define IDC_RED                         2226
#define IDC_BLUE                        2227
#define IDC_GREEN                       2228
#define IDC_SAVECOLOR                   2229
#define IDC_SETCOLOR                    2229
#define IDC_FILES_ICO                   2231
#define IDC_NICK_FRM                    2232
//#define IDC_TOOLTIPDELAY_LBL            2233
#define IDC_CAPACITIES_FRM              2234
#define IDC_LIMITS_FRM                  2235
//#define IDC_DCAP_LBL                    2236
//#define IDC_UCAP_LBL                    2237
//#define IDC_DLIMIT_LBL                  2238
//#define IDC_ULIMIT_LBL                  2239
#define IDC_CLIENTPORT_FRM              2240
#define IDC_MAXCONN_FRM                 2241
#define IDC_MAXSRC_FRM                  2242
//#define IDC_MAXSRCHARD_LBL              2243
#define IDC_MAXSRCSOFT_LBL              2244
#define IDC_MAXSRCASK_LBL               2244
#define IDC_MISC_FRM                    2245
#define IDC_MAXSRCUDP_LBL               2245
//#define IDC_RETRIES_LBL                 2249
#define IDC_INCOMING_FRM                2250
#define IDC_TEMP_FRM                    2251
#define IDC_SHARED_FRM                  2252
#define IDC_ICH_FRM                     2253
#define IDC_SEEMYSHARE_FRM              2254
#define IDC_MESSAGES_LBL                2255
#define IDC_SETUPBUTTON                 2256
#define IDD_PPG_NOTIFY                  2256
//#define IDC_SLIDERINFO3                 2257
//#define IDC_EDIT_TBN_WAVFILE            2257
#define IDD_PPG_IRC                     2257
//#define IDC_GRAPHS                      2258
//#define IDC_CB_TBN_ONLOG                2258
#define IDD_WIZARD                      2258
// #define IDC_STREE                       2259
//#define IDC_CB_TBN_POP_ALWAYS           2259
#define IDD_PPG_TWEAKS                  2259
//#define IDC_KBS1                        2260
#define IDD_ADDFRIEND                   2260
//#define IDC_KBS2                        2261
//#define IDC_BTN_BROWSE_WAV              2261
//#define IDC_KBS3                        2262
//#define IDC_KBS4                        2263
//#define IDC_CB_TBN_ONDOWNLOAD           2264
#define IDC_IRC_SERVER_FRM              2264
//#define IDC_CB_TBN_ONCHAT               2265
#define IDC_IRC_NICK_FRM                2265
//#define IDC_CB_TBN_USESOUND             2266
#define IDC_IRC_FILTER_FRM              2266
#define IDC_IRC_NAME_TEXT               2267
//#define IDC_CB_TBN_IMPORTATNT           2267
#define IDC_IRC_MINUSER_TEXT            2268
#define IDC_IRC_PERFORM_FRM             2269
//#define IDC_CB_TBN_ONNEWVERSION         2270
#define IDC_TAB2                        2271
#define IDC_WONLOST                     2272
//#define IDC_FILENAME                    2273
#define IDC_IRC_ACCEPTLINKS             2274
//#define IDC_FD_SN                       2276
//#define IDC_PREFCOLORS                  2278
#define IDC_IRC_INFOMESSAGE             2279
#define IDC_EDIT1                       2280
#define IDC_TRUEUPLOAD_BOX              2280
#define IDC_WIZ_TRUEUPLOAD_BOX          2280
//#define IDC_UDPPORT                     2280
#define IDC_USERNAME                    2280
//#define IDC_CMT_TEXT                    2280
#define IDC_MAXSRCPERFILESOFT           2280
#define IDC_MAXSRCEPERFILESOFT          2280
#define IDC_DEBUG_LOG                   2280
//#define IDC_TITLE                       2280
#define IDC_FILTERLEVEL                 2280
#define IDC_TEXT                        2280
#define IDC_WSPORT                      2280
//#define IDC_WIZARD                      2281
#define IDC_USERHASH                    2281
//#define IDC_INCOMING                    2281
#define IDC_WSPASS                      2281
#define IDC_OS_FRAME                    2282
#define IDC_IP                          2282
#define IDC_INCOMING2                   2282
//#define IDC_COMMENT                     2282
#define IDC_WSPASS2                     2282
#define IDC_WSPASSLOW                   2282
#define IDC_HOTBUTTON_FRAME             2283
#define IDC_BROADBAND_TEXT              2284
#define IDC_256K_BUTTON                 2285
#define IDC_MODEM_TEXT                  2286
#define IDC_CUSTOM_FRAME                2287
#define IDC_WIZ_TRUEUPLOAD_TEXT         2288
#define IDC_TRUEDOWNLOAD_TEXT           2289
#define IDC_WIZ_OS_FRAME                2290
#define IDC_WIZ_CONCURENTDOWN_FRAME     2291
#define IDC_WIZ_LOWDOWN_RADIO           2292
#define IDC_WIZ_MEDIUMDOWN_RADIO        2293
#define IDC_WIZ_HIGHDOWN_RADIO          2294
#define IDC_WIZ_128K_BUTTON             2295
#define IDC_WIZ_CUSTOM_FRAME            2296
#define IDC_WIZ_TRUEDOWNLOAD_TEXT       2297
#define IDC_WIZ_HOTBUTTON_FRAME         2298
#define IDC_FD_XI3                      2299
#define IDC_FD_XI4                      2300
#define IDC_WIZ_KBSSWITCH_BUTTON        2301
#define IDC_PROVIDERS                   2302
#define IDC_KBITS                       2303
#define IDC_KBYTES                      2304
//#define IDC_FILEINFO                    2305
//#define IDC_FILESIZE                    2306
#define IDC_UNIT                        2307
#define IDC_CTINFO                      2308
//#define IDC_MAXCONLABEL                 2310
//#define IDC_MAXCON5SECLABEL             2311
//#define IDC_WARNING                     2312
#define IDC_PWT_NETWORK                 2313
#define IDC_INFO1                       2314
#define IDC_INFO2                       2315
//#define IDCOK                           2316
//#define IDCCANCEL                       2317
#define IDC_CMT_LQUEST                  2318
#define IDC_CMT_LAIDE                   2319
//#define IDCREF                          2319
#define IDC_RATEQUEST                   2320
//#define IDC_LST                         2320
#define IDC_RATEHELP                    2321
//#define IDC_RATELIST                    2322
#define IDC_TAB3                        2324
//#define IDC_3DDEPTH                     2326
//#define IDC_3DDEP                       2327
//#define IDC_MANUALSERVERHIGHPRIO        2328
//#define IDC_DAYS                        2328
#define IDC_ROUNDBIT                    2329
//#define IDC_ROUND                       2329
#define IDC_STARTUP                     2330
#define IDC_STATICVIDEOPLAYER           2332
#define IDC_STATIC_EMPTY                2333
//#define IDC_FILEBUFFERSIZE              2334
//#define IDC_FILEBUFFERSIZE_STATIC       2335
//#define IDC_QUEUESIZE_STATIC            2336
//#define IDC_QUEUESIZE                   2337
//#define IDC_CHECKDAYS                   2338
#define IDC_MSG                         2339
//#define IDC_FILTERLABEL                 2340
//#define IDC_DAP                         2341
#define IDC_DLTAB                       2343
//#define IDC_STATIC_INCOMING             2344
//#define IDC_STATIC_TITLE                2345
//#define IDC_STATIC_COMMENT              2346
#define IDC_STATIC_DLTOof               2347
#define IDC_DOWNLOADINGLIST             2348
#define IDC_CONNECTEDLIST               2349
#define IDC_FILTERSERVERBYIPFILTER      2350
#define IDC_STATIC_FILTERLEVEL          2351
#define IDC_STATIC_IPFILTER             2352
#define IDC_STATIC_COMMENTS             2353
#define IDC_CLIENTLIST                  2354
#define IDC_STATIC_FILTERLEVEL2         2354
#define IDC_EDITFILTER                  2355
//#define IDC_CATCOLOR                    2356
#define IDC_FILTERCOMMENTSLABEL         2357
#define IDC_WATCHCB                     2358
//#define IDC_STATIC_COLOR                2359
#define IDC_CATTAB2                     2360
#define IDC_USECREDITSYSTEM             2361
#define IDC_STATIC_CPUMEM               2362
#define IDC_LOG2DISKFRAME               2363
#define IDC_LABEL                       2364
#define IDC_IBLABEL                     2364
#define IDC_WSPORT_LBL                  2365
#define IDC_WSPASS_LBL                  2367
#define IDD_PPG_WEBSRV                  2368
#define IDC_WSPASS_LBL2                 2368
#define IDI_PREF_WEBSERVER              2369
#define IDC_STATIC_LOWUSER              2370
#define IDC_STATIC_ADMIN                2371
#define IDC_STATIC_GENERAL              2372
#define IDC_DATETIMEPICKER1             2373
#define IDC_STATIC_S_TITLE              2374
#define IDC_STATIC_S_TIME               2375
#define IDC_STATIC_S_ACTION             2376
#define IDC_DATETIMEPICKER2             2377
#define IDC_TIMESEL                     2378
#define IDC_STATIC_DETAILS              2379
#define IDC_NEW                         2380
#define IDC_MSGONLYSEC                  2381
//#define IDC_RECONN                      2383
#define IDC_SNAME                       2384
#define IDC_IRC_TIMESTAMP               2385
//#define IDC_UAP                         2386
//#define IDC_SMARTIDCHECK                2387
//#define IDC_VIDEOBACKUP                 2389
#define IDC_REPAINT                     2389
#define IDC_LOG2DISK                    2390
#define IDC_WSENABLED                   2391
#define IDC_ENABLE                      2392
#define IDC_MSGONLYFRIENDS              2393
//#define IDC_SELTEMPDIR                  2394
//#define IDC_FD_X2                       2395
//#define IDC_FD_X3                       2396
//#define IDC_FD_X4                       2397
#define IDC_SCHEDACTION                 2398
#define IDC_SCHEDLIST                   2399
#define IDC_EDIT2                       2400
#define IDC_TMPLPATH                    2400
#define IDC_TEMPLATE                    2401
#define IDD_PPG_PROXY                   2402
#define IDC_ENABLEPROXY                 2403
#define IDC_ENABLEAUTH                  2404
#define IDC_PROXYTYPE                   2405
#define IDC_PROXYNAME                   2406
#define IDC_PROXYPORT                   2407
#define IDC_USERNAME_A                  2408
#define IDC_PASSWORD                    2409
#define IDC_PROXYTYPE_LBL               2410
#define IDC_PROXYNAME_LBL               2411
#define IDC_PROXYPORT_LBL               2412
#define IDC_USERNAME_LBL                2414
#define IDC_PASSWORD_LBL                2415
#define IDC_AUTH_LBL                    2416
#define IDC_ASCWOP                      2417
#define IDI_PREF_PROXY                  2418
#define IDC_SD_MB1                      2419
#define IDC_SD_MB2                      2420
#define IDC_AUTH_LBL2                   2421
#define IDC_WIZ1_TITLE                  2422
#define IDC_WIZ1_ACTIONS                2423
#define IDC_WIZ1_BTN_HINT               2424
#define IDI_ICON8                       2425
#define IDI_PREF_DISPLAY                2426
#define IDD_PPG_DISPLAY                 2427
#define IDD_CAT                         2428
#define IDR_TASKBAR                     2429
#define IDI_PREF_SECURITY               2430
#define IDD_PPG_SECURITY                2431
#define IDC_IRC_EMULEPROTO_INFOMESSAGE  2432
//#define IDC_STATIC_PRIO                 2433
//#define IDC_PRIOCOMBO                   2434
#define IDR_WIZ1_WATERMARK              2435
#define IDR_WIZ1_HEADER                 2436
#define IDC_STARTNEXTFILECAT            2437
#define IDD_WIZ1_WELCOME                2438
#define IDD_WIZ1_GENERAL                2439
#define IDD_WIZ1_DL_PRIO                2440
#define IDD_WIZ1_UL_PRIO                2441
#define IDD_WIZ1_UPLOAD                 2442
#define IDD_WIZ1_SERVER                 2443
#define IDD_WIZ1_END                    2444
#define IDC_STATIC_FILTER               2445
#define IDC_LAST_SEEN_LABEL             2446
#define IDS_CONFIRMATION		2447
#define IDS_CATEGORY_DELETE		2448
#define IDS_TEXT			2449
#define IDS_OUT_OF_SPACE		2450

/* Razor 1a - Modif by MikaelB */
#define IDS_DROP_NO_NEEDED_SOURCES           5721
#define IDS_DROP_FULL_QUEUE_SOURCES          5722
#define IDS_DROP_HIGH_QUEUE_RATING_SOURCES   5723
#define IDS_CLEAN_UP_SOURCES                 5724
#define IDS_SWAP_A4AF_TO_THIS                5725
#define IDS_SWAP_A4AF_TO_THIS_AUTO           5726
#define IDS_SWAP_A4AF_TO_ANY_OTHER           5727
#define IDS_EXTENDED                         5728
/* End modif */
  
#define IDD_PPG_SOURCES_DROPPING 12743
#define IDD_PPG_GUI_TWEAKS 12744
//#define IDC_ENABLE_AUTO_NNS 12745
#define IDC_AUTO_NNS_TIMER_TEXT 12746
#define IDC_AUTO_NNS_TIMER 12747
#define IDC_AUTO_NNS_TIMER_HELP 12748
//#define IDC_ENABLE_AUTO_FQS 12749
#define IDC_AUTO_FQS_TIMER_TEXT 12750
#define IDC_AUTO_FQS_TIMER 12751
#define IDC_AUTO_FQS_TIMER_HELP 12752
//#define IDC_ENABLE_AUTO_HQRS 12753
#define IDC_AUTO_HQRS_TIMER_TEXT 12754
#define IDC_AUTO_HQRS_TIMER 12755
#define IDC_AUTO_HQRS_TIMER_HELP 12756
#define IDC_AUTO_HQRS_HQR_TEXT 12757
#define IDC_AUTO_HQRS_HQR 12758
#define IDC_AUTO_HQRS_HQR_HELP 12759
#define IDC_ENABLE_DYNUPLOAD 10002
#define IDC_PING_URL 10003
#define IDC_RESPONSE_VALUE 10004
#define IDC_PING_RADIO 10005
#define IDS_CATCOLOR 10006
//#define IDC_SELECTED_THEME 10006
//#define IDC_USE_FONT 10007
//#define IDC_SELECTED_FONT 10008
//#define IDC_FONT_BUTTON 10009
//#define IDC_PREVIEW_THEME 10010
//#define IDC_APPLY_THEME 10011
//#define IDC_ENABLE_AUTO_ARRANGING 10012
#define IDC_ARRANGE_CHOICE 10013
		
// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        1006
#define _APS_NEXT_COMMAND_VALUE         32771
#define _APS_NEXT_CONTROL_VALUE         2447
#define _APS_NEXT_SYMED_VALUE           106
#endif
#endif

#endif // RESOURCE_H
