// src/timestamp-sys.h.  Generated from sys.h.in by configure.
//
// System control file.
// This file is part of the aMule project
//
// Copyright (C) 2003, by Carlo Wood
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef SYS_H
#define SYS_H

//
// The build system forgets to add the correct CXXFLAGS (like for
// example -pthread for g++).  Therefore define this here instead.
// Threaded application should this always have defined because
// included library headers might depend on it.
//
#ifndef _REENTRANT
#define _REENTRANT
#endif

//
// Some OS use _THREAD_SAFE instead of _REENTRANT
//
#ifdef _REENTRANT
#define _THREAD_SAFE
#endif

#ifndef __linux__
#include <sys/types.h>
#endif

//
// When libcwd is used, include its sys.h too.
//
#ifdef CWDEBUG
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <libcwd/sys.h>
#endif

#endif // SYS_H
