//------------------------------------------------------------------------------
// Header generated by wxDesigner from file: muuli.wdr
// Do not modify this file, all changes will be lost!
//------------------------------------------------------------------------------

#ifndef __WDR_muuli_H__
#define __WDR_muuli_H__

#if defined(__GNUG__) && !defined(__APPLE__)
    #pragma interface "muuli_wdr.h"
#endif

// Include wxWindows' headers

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <wx/image.h>
#include <wx/statline.h>
#include <wx/spinbutt.h>
#include <wx/spinctrl.h>
#include <wx/splitter.h>
#include <wx/listctrl.h>
#include <wx/treectrl.h>
#include <wx/notebook.h>
#include <wx/grid.h>

// Declare window functions

#define ID_TEXT 10000
#define ID_GNOME2 10001
#define ID_KDE3 10002
#define ID_KDE2 10003
#define ID_NOSYSTRAY 10004
#define ID_OK 10005
wxSizer *desktopDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

extern wxSizer *buttonToolbar;
extern wxSizer *contentSizer;
extern wxSizer *s_fed2klh;
#define ID_TEXTCTRL 10006
#define ID_BUTTON_FAST 10007
#define ID_LINE 10008
#define ID_STATICBITMAP 10009
wxSizer *muleDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_SRV_SPLITTER 10010
wxSizer *serverListDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

extern wxSizer *IDC_SEARCH_FRM;
#define IDC_MSTATIC3 10011
#define IDC_SEARCHNAME 10012
#define IDC_SEARCH_RESET 10013
#define IDC_SEARCHWEBNAME 10014
#define ID_WEBSEARCH_SUBMIT 10015
#define IDC_MSTATIC4 10016
#define IDC_TypeSearch 10017
#define IDC_EDITSEARCHMIN 10018
#define IDC_SEARCHMINSIZE 10019
#define IDC_EDITSEARCHMAX 10020
#define IDC_SEARCHMAXSIZE 10021
#define IDC_EDITSEARCHEXTENSION 10022
#define IDC_SEARCHEXTENSION 10023
#define IDC_EDITSEARCHAVAIBILITY 10024
#define IDC_SEARCHAVAIL 10025
#define IDC_SGLOBAL 10026
#define IDC_STARTS 10027
#define IDC_CANCELS 10028
#define IDC_CLEARALL 10029
#define IDC_RESULTS_LBL 10030
#define ID_NOTEBOOK 10031
#define IDC_SDOWNLOAD 10032
wxSizer *searchDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_BTNCLRCOMPL 10033
#define ID_CATEGORIES 10034
#define ID_DLOADLIST 10035
wxSizer *transferTopPane( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

extern wxSizer *queueSizer;
#define ID_BTNSWITCHUP 10036
#define ID_QUEUELIST 10037
#define ID_UPLOADLIST 10038
wxSizer *transferBottomPane( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_FRIENDLIST 10039
#define IDC_CHATSELECTOR 10040
#define ID_HTMLWIN 10041
#define IDC_CMESSAGE 10042
#define IDC_CSEND 10043
#define IDC_CCLOSE 10044
wxSizer *messagePage( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

extern wxSizer *IDC_FD_X0;
#define IDC_FD_X1 10045
#define IDC_FNAME 10046
#define IDC_FD_X2 10047
#define IDC_METFILE 10048
#define IDC_FD_X3 10049
#define IDC_FHASH 10050
#define IDC_FD_X4 10051
#define IDC_FSIZE 10052
#define IDC_FD_X5 10053
#define IDC_PFSTATUS 10054
#define IDC_FD_X15 10055
#define IDC_LASTSEENCOMPL 10056
extern wxSizer *IDC_FD_X6;
#define IDC_FD_X7 10057
#define IDC_SOURCECOUNT 10058
#define IDC_FD_X10 10059
#define IDC_FD_X9 10060
#define IDC_PARTCOUNT 10061
#define IDC_FD_X11 10062
#define IDC_FD_X13 10063
#define IDC_DATARATE 10064
#define IDC_FD_X14 10065
#define IDC_TRANSFERED 10066
#define IDC_FD_X12 10067
#define IDC_SOURCECOUNT2 10068
#define IDC_PARTAVAILABLE 10069
#define IDC_COMPLSIZE 10070
#define IDC_PROCCOMPL 10071
extern wxSizer *IDC_FD_ICH;
#define IDC_FD_LSTATS1 10072
#define IDC_FD_STATS1 10073
#define IDC_FD_LSTATS2 10074
#define IDC_FD_STATS2 10075
#define IDC_FD_LSTATS3 10076
#define IDC_FD_STATS3 10077
extern wxSizer *IDC_FD_SN;
#define IDC_LISTCTRLFILENAMES 10078
#define IDC_TAKEOVER 10079
#define IDC_BUTTONSTRIP 10080
#define IDC_FILENAME 10081
#define IDC_RENAME 10082
#define IDC_CMTBT 10083
#define ID_CLOSEWNDFD 5100
wxSizer *fileDetails( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_CMT_TEXT 10084
#define IDC_FC_CLEAR 10085
#define IDC_RATELIST 10086
#define IDCOK 10087
#define IDCCANCEL 10088
wxSizer *commentDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_LST 10089
#define IDC_CMSTATUS 10090
#define IDCREF 10091
wxSizer *commentLstDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_ANIMATE 10092
wxSizer *downloadDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_IPADDRESS 10093
#define ID_IPORT 10094
#define ID_USERNAME 10095
#define ID_USERHASH 10096
#define ID_ADDFRIEND 10097
#define ID_CLOSEDLG 10098
wxSizer *addFriendDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_BTNRELSHARED 10099
#define ID_SHFILELIST 10100
#define IDC_SREQUESTED 10101
#define ID_GAUGE 10102
#define IDC_SREQUESTED2 10103
#define IDC_SACCEPTED 10104
#define IDC_SACCEPTED2 10105
#define IDC_STRANSFERED 10106
#define IDC_STRANSFERED2 10107
wxSizer *sharedfilesDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_SEARCHLISTCTRL 10108
#define ID_SEARCHPROGRESS 10109
wxSizer *searchPage( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

extern wxSizer *testSizer;
#define ID_DLOADSCOPE 10110
#define IDC_C0 10111
#define IDC_C0_3 10112
#define IDC_C0_2 10113
#define ID_OTHERS 10114
#define IDC_S3 10115
#define IDC_S0 10116
#define ID_ACTIVEC 10117
#define IDC_S1 10118
#define ID_ULOADSCOPE 10119
#define IDC_C1 10120
#define IDC_C1_3 10121
#define IDC_C1_2 10122
#define ID_TREECTRL 10123
wxSizer *statsDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_DNAME 10124
#define ID_DHASH 10125
#define ID_DSOFT 10126
#define ID_DIP 10127
#define ID_DSIP 10128
#define ID_DVERSION 10129
#define ID_DID 10130
#define ID_DSNAME 10131
#define ID_DDOWNLOADING 10132
#define ID_DDOWN 10133
#define ID_DAVDR 10134
#define ID_DDOWNTOTAL 10135
#define ID_DDUP 10136
#define ID_DAVUR 10137
#define ID_DUPTOTAL 10138
#define ID_DRATIO 10139
#define ID_DRATING 10140
#define ID_DSCORE 10141
#define ID_CLOSEWND 10142
wxSizer *clientDetails( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_NICK 10143
#define IDC_LANGUAGE 10144
#define IDC_3DDEP 10145
#define IDC_3DDEPTH 10146
#define IDC_FLAT 10147
#define IDC_ROUND 10148
#define IDC_BEEPER 10149
#define IDC_BRINGTOFOREGROUND 10150
#define IDC_DBLCLICK 10151
#define IDC_MINTRAY 10152
#define IDC_ONLINESIG 10153
#define IDC_EXIT 10154
#define IDC_TOOLTIPDELAY_LBL 10155
#define IDC_TOOLTIPDELAY 10156
#define ID_DESKTOPMODE 10157
#define IDC_DAYS 10158
#define IDC_CHECK4UPDATE 10159
#define IDC_CHECKDAYS 10160
#define IDC_SPLASHON 10161
#define IDC_STARTMIN 10162
#define IDC_FED2KLH 10163
wxSizer *PreferencesGeneralTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_DCAP_LBL 10164
#define IDC_DOWNLOAD_CAP 10165
#define IDC_KBS2 10166
#define IDC_UCAP_LBL 10167
#define IDC_UPLOAD_CAP 10168
#define IDC_KBS3 10169
#define IDC_DLIMIT_LBL 10170
#define IDC_MAXDOWN 10171
#define IDC_KBS1 10172
#define IDC_ULIMIT_LBL 10173
#define IDC_MAXUP 10174
#define IDC_KBS4 10175
#define IDC_SLOTALLOC 10176
#define IDC_PORT 10177
#define IDC_UDPPORT 10178
#define IDC_UDPDISABLE 10179
#define IDC_MAXSRCHARD_LBL 10180
#define IDC_MAXSOURCEPERFILE 10181
#define IDC_MAXCONLABEL 10182
#define IDC_MAXCON 10183
#define IDC_AUTOCONNECT 10184
#define IDC_RECONN 10185
#define IDC_SHOWOVERHEAD 10186
wxSizer *PreferencesConnectionTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_REMOVEDEAD 10187
#define IDC_SERVERRETRIES 10188
#define IDC_RETRIES_LBL 10189
#define IDC_AUTOSERVER 10190
#define IDC_EDITADR 10191
#define IDC_FILTER 10192
#define IDC_UPDATESERVERCONNECT 10193
#define IDC_UPDATESERVERCLIENT 10194
#define IDC_SCORE 10195
#define IDC_SMARTIDCHECK 10196
#define IDC_SAFESERVERCONNECT 10197
#define IDC_AUTOCONNECTSTATICONLY 10198
#define IDC_MANUALSERVERHIGHPRIO 10199
wxSizer *PreferencesServerTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_SEESHARE1 10200
#define IDC_SEESHARE2 10201
#define IDC_SEESHARE3 10202
#define IDC_ICH 10203
#define IDC_ADDNEWFILESPAUSED 10204
#define IDC_DAP 10205
#define IDC_PREVIEWPRIO 10206
#define IDC_UAP 10207
#define IDC_FULLCHUNKTRANS 10208
#define IDC_STARTNEXTFILE 10209
wxSizer *PreferencesFilesTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_INCFILES 10210
#define IDC_SELINCDIR 10211
#define IDC_TEMPFILES 10212
#define IDC_SELTEMPDIR 10213
#define IDC_SHARESELECTOR 10214
#define IDC_VIDEOPLAYER 10215
#define IDC_BROWSEV 10216
#define IDC_VIDEOBACKUP 10217
wxSizer *PreferencesDirectoriesTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_SLIDERINFO 10218
#define IDC_SLIDER 10219
#define IDC_SLIDERINFO3 10220
#define IDC_SLIDER3 10221
#define IDC_PREFCOLORS 10222
#define IDC_COLORSELECTOR 10223
#define IDC_COLOR_BUTTON 10224
#define IDC_SLIDERINFO2 10225
#define IDC_SLIDER2 10226
wxSizer *PreferencesStatisticsTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_CB_TBN_USESOUND 10227
#define IDC_EDIT_TBN_WAVFILE 10228
#define IDC_BTN_BROWSE_WAV 10229
#define IDC_CB_TBN_ONLOG 10230
#define IDC_CB_TBN_ONCHAT 10231
#define IDC_CB_TBN_POP_ALWAYS 10232
#define IDC_CB_TBN_ONDOWNLOAD 10233
#define IDC_CB_TBN_ONNEWVERSION 10234
#define IDC_CB_TBN_IMPORTATNT 10235
#define IDC_SENDMAIL 10236
#define IDC_SMTP 10237
#define IDC_EMAIL 10238
wxSizer *PreferencesNotifyTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_WARNING 10239
#define IDC_STATIC 10240
#define IDC_MAXCON5SECLABEL 10241
#define IDC_MAXCON5SEC 10242
#define IDC_VERBOSE 10243
#define IDC_AUTOTAKEED2KLINKS 10244
#define IDC_SHOWRATEONTITLE 10245
#define IDC_FILEBUFFERSIZE_STATIC 10246
#define IDC_FILEBUFFERSIZE 10247
#define IDC_QUEUESIZE_STATIC 10248
#define IDC_QUEUESIZE 10249
#define IDC_SERVERKEEPALIVE_LABEL 10250
#define IDC_SERVERKEEPALIVE 10251
wxSizer *PreferencesaMuleTweaksTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_ENABLE_AUTO_ARRANGING 10252
#define ID_RADIOBUTTON 10253
#define IDC_SELECTED_THEME 10254
#define IDC_USE_FONT 10255
#define IDC_SELECTED_FONT 10256
#define IDC_FONT_BUTTON 10257
#define IDC_PREVIEW_THEME 10258
#define IDC_APPLY_THEME 10259
wxSizer *PreferencesGuiTweaksTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_ENABLE_AUTO_NNS 10260
#define IDC_AUTO_NNS_EXTENDED_RADIO 10261
#define IDC_ENABLE_AUTO_FQS 10262
#define IDC_ENABLE_AUTO_HQRS 10263
#define IDC_HQR_VALUE 10264
#define IDC_AUTO_DROP_TIMER 10265
wxSizer *PreferencesSourcesDroppingTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_ENABLE_WEB 10266
#define IDC_WEB_PORT 10267
#define IDC_WEB_REFRESH_TIMEOUT 10268
#define IDC_WEB_GZIP 10269
#define IDC_ENABLE_WEB_LOW 10270
#define IDC_WEB_PASSWD 10271
#define IDC_WEB_PASSWD_LOW 10272
#define IDC_EXT_CONN_ACCEPT 10273
#define IDC_EXT_CONN_USETCP 10274
#define IDC_EXT_CONN_TCP_PORT 10275
#define IDC_EXT_CONN_PASSWD_ENABLE 10276
#define IDC_EXT_CONN_PASSWD 10277
wxSizer *PreferencesRemoteControlsTab( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_NOTEBOOK_TOP 10278
#define ID_PREFS_OK_TOP 10279
#define ID_PREFS_CANCEL_TOP 10280
wxSizer *preferencesDlgTop( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define IDC_STATIC_TITLE 10281
#define IDC_TITLE 10282
#define IDC_STATIC_COMMENT 10283
#define IDC_COMMENT 10284
#define IDC_STATIC_INCOMING 10285
#define IDC_INCOMING 10286
#define IDC_BROWSE 10287
#define IDC_STATIC_PRIO 10288
#define IDC_PRIOCOMBO 10289
#define IDC_STATIC_COLOR 10290
#define ID_BOX_CATCOLOR 10291
#define IDC_CATCOLOR 10292
wxSizer *CategoriesEditWindow( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_SPLATTER 10293
wxSizer *transferDlg( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_SERVERINFO 10294
wxSizer *ServerInfoLog( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_LOGVIEW 10295
#define ID_BTN_RESET 10296
wxSizer *aMuleLog( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_UPDATELIST 10297
#define IDC_SERVERLISTURL 10298
#define IDC_SERVERNAME 10299
#define IDC_IPADDRESS 10300
#define IDC_SPORT 10301
#define ID_ADDTOLIST 10302
#define ID_SERVERLIST 10303
wxSizer *serverListDlgUp( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_SRVLOG_NOTEBOOK 10304
wxSizer *serverListDlgDown( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

#define ID_NOTEBOOK_LEFT 10305
#define ID_PREFS_OK_LEFT 10306
#define ID_PREFS_CANCEL_LEFT 10307
wxSizer *preferencesDlgLeft( wxWindow *parent, bool call_fit = TRUE, bool set_sizer = TRUE );

// Declare menubar functions

// Declare toolbar functions

#define ID_BUTTONCONNECT 10308
#define ID_BUTTONSERVERS 10309
#define ID_BUTTONSEARCH 10310
#define ID_BUTTONTRANSFER 10311
#define ID_BUTTONSHARED 10312
#define ID_BUTTONMESSAGES 10313
#define ID_BUTTONSTATISTICS 10314
#define ID_BUTTONPREFERENCES 10315
#define ID_BUTTONNEWPREFERENCES 10316
void muleToolbar( wxToolBar *parent );

// Declare bitmap functions

wxBitmap clientImages( size_t index );

wxBitmap dlStatusImages( size_t index );

wxBitmap connImages( size_t index );

wxBitmap moreImages( size_t index );

wxBitmap amuleSpecial( size_t index );

wxBitmap connButImg( size_t index );

wxBitmap amuleDlgImages( size_t index );

#endif

// End of generated file
