// This file is part of the aMule Project
//
// Copyright (c) 2003-2004 aMule Project ( http://www.amule-project.net )
// Copyright (C) 2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef TYPES_H
#define TYPES_H

#include <inttypes.h>		// Needed for int8_t, int16_t, int32_t, int64_t, uint8_t, uint16_t and uint64_t

// Backwards compatibility with emule.
// Note that the int* types are indeed unsigned.
typedef uint8_t			int8;
typedef uint8_t			uint8;
typedef uint16_t		int16;
typedef uint16_t		uint16;
typedef uint32_t		int32;
typedef uint32_t		uint32;
typedef uint64_t		int64;
typedef uint64_t		uint64;
typedef int8_t			sint8;
typedef int16_t			sint16;
typedef int32_t			sint32;
typedef int64_t			sint64;
typedef unsigned long		ulong;
typedef unsigned short		ushort;
typedef unsigned char		uchar;

#endif // TYPES_H
